@@ -14,17 +14,22 @@ Copyright (C) 2007-2022 David Aguilar and contributors
 
 try:
     from qtpy import QtCore
-except ImportError:
+except ImportError as error:
     sys.stderr.write(
         """
-You do not seem to have PyQt (or PySide) and qtpy installed.
-Please install the PyQt (or PySide) and qtpy into your Python environment.
+Your Python environment does not have qtpy and PyQt (or PySide).
+The following error was encountered when importing "qtpy":
 
-For example, on a Debian/Ubuntu system you can install these using apt:
+    ImportError: {err}
 
-    sudo apt install python3-pyqt5 python3-pyqt5.qtwebkit python3-qtpy
+Install qtpy and PyQt (or PySide) into your Python environment.
+On a Debian/Ubuntu system you can install these modules using apt:
 
-"""
+    sudo apt install python3-pyqt5 python3-pyqt5.qtwebengine python3-qtpy
+
+""".format(
+            err=error
+        )
     )
     sys.exit(1)
 
@@ -141,22 +146,6 @@ def setup_environment():
     # have a chance to explain our merges.
     compat.setenv('GIT_MERGE_AUTOEDIT', 'no')
 
-    # Gnome3 on Debian has XDG_SESSION_TYPE=wayland and
-    # XDG_CURRENT_DESKTOP=GNOME, which Qt warns about at startup:
-    #
-    #   Warning: Ignoring XDG_SESSION_TYPE=wayland on Gnome.
-    #   Use QT_QPA_PLATFORM=wayland to run on Wayland anyway.
-    #
-    # This annoying, so we silence the warning.
-    # We'll need to keep this hack here until a future version of Qt provides
-    # Qt Wayland widgets that are usable in gnome-shell.
-    # Cf. https://bugreports.qt.io/browse/QTBUG-68619
-    if (
-        core.getenv('XDG_CURRENT_DESKTOP', '') == 'GNOME'
-        and core.getenv('XDG_SESSION_TYPE', '') == 'wayland'
-    ):
-        compat.unsetenv('XDG_SESSION_TYPE')
-
 
 def get_icon_themes(context):
     """Return the default icon theme names"""
@@ -192,6 +181,7 @@ class ColaApplication(object):
         icons.install(icon_themes or get_icon_themes(context))
 
         self.context = context
+        self.theme = None
         self._install_hidpi_config()
         self._app = ColaQApplication(context, list(argv))
         self._app.setWindowIcon(icons.cola())
@@ -203,6 +193,7 @@ class ColaApplication(object):
         if theme_str is None:
             theme_str = self.context.cfg.get('cola.theme', default='default')
         theme = themes.find_theme(theme_str)
+        self.theme = theme
         self._app.setStyleSheet(theme.build_style_sheet(self._app.palette()))
         if theme_str != 'default':
             self._app.setPalette(theme.build_palette(self._app.palette()))
@@ -220,6 +211,10 @@ class ColaApplication(object):
         """QApplication::desktop() pass-through"""
         return self._app.desktop()
 
+    def palette(self):
+        """QApplication::palette() pass-through"""
+        return self._app.palette()
+
     def start(self):
         """Wrap exec_() and start the application"""
         # Defer connection so that local cola.inotify is honored
