@@ -127,6 +127,11 @@ def mkpath(path, encoding=None):
     return encode(path, encoding=encoding)
 
 
+def decode_seq(seq, encoding=None):
+    """Decode a sequence of values"""
+    return [decode(x, encoding=encoding) for x in seq]
+
+
 def list2cmdline(cmd):
     return subprocess.list2cmdline([decode(c) for c in cmd])
 
@@ -342,7 +347,7 @@ def _win32_find_exe(exe):
 
 
 # Portability wrappers
-if sys.platform == 'win32' or sys.platform == 'cygwin':
+if sys.platform in {'win32', 'cygwin'}:
     fork = _fork_win32
 else:
     fork = _fork_posix
@@ -352,24 +357,24 @@ def _decorator_noop(x):
     return x
 
 
-def wrap(action, fn, decorator=None):
+def wrap(action, func, decorator=None):
     """Wrap arguments with `action`, optionally decorate the result"""
     if decorator is None:
         decorator = _decorator_noop
 
-    @functools.wraps(fn)
+    @functools.wraps(func)
     def wrapped(*args, **kwargs):
-        return decorator(fn(action(*args, **kwargs)))
+        return decorator(func(action(*args, **kwargs)))
 
     return wrapped
 
 
-def decorate(decorator, fn):
-    """Decorate the result of `fn` with `action`"""
+def decorate(decorator, func):
+    """Decorate the result of `func` with `action`"""
 
-    @functools.wraps(fn)
+    @functools.wraps(func)
     def decorated(*args, **kwargs):
-        return decorator(fn(*args, **kwargs))
+        return decorator(func(*args, **kwargs))
 
     return decorated
 
@@ -391,9 +396,29 @@ def guess_mimetype(filename):
 
 
 def xopen(path, mode='r', encoding=None):
+    """Open a file with the specified mode and encoding
+
+    The path is decoded into unicode on Windows and encoded into bytes on Unix.
+    """
+    # pylint: disable=unspecified-encoding
     return open(mkpath(path, encoding=encoding), mode)
 
 
+def open_append(path, encoding=None):
+    """Open a file for appending in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'a', encoding='utf-8')
+
+
+def open_read(path, encoding=None):
+    """Open a file for reading in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'rt', encoding='utf-8')
+
+
+def open_write(path, encoding=None):
+    """Open a file for writing in utf-8 text mode"""
+    return open(mkpath(path, encoding=encoding), 'wt', encoding='utf-8')
+
+
 def print_stdout(msg, linesep='\n'):
     msg = msg + linesep
     if PY2:
@@ -450,16 +475,27 @@ def _find_executable(executable, path=None):
         executable = executable + '.exe'
 
     if not os.path.isfile(executable):
-        for p in paths:
-            f = os.path.join(p, executable)
-            if os.path.isfile(f):
+        for dirname in paths:
+            filename = os.path.join(dirname, executable)
+            if os.path.isfile(filename):
                 # the file exists, we have a shot at spawn working
-                return f
+                return filename
         return None
 
     return executable
 
 
+def sync():
+    """Force writing of everything to disk. No-op on systems without os.sync()"""
+    if hasattr(os, 'sync'):
+        os.sync()
+
+
+def rename(old, new):
+    """Rename a path. Transform arguments to handle non-ascii file paths"""
+    os.rename(mkpath(old), mkpath(new))
+
+
 if PY2:
     find_executable = wrap(mkpath, _find_executable, decorator=decode)
 else:
@@ -467,6 +503,7 @@ else:
 isdir = wrap(mkpath, os.path.isdir)
 isfile = wrap(mkpath, os.path.isfile)
 islink = wrap(mkpath, os.path.islink)
+listdir = wrap(mkpath, os.listdir, decorator=decode_seq)
 makedirs = wrap(mkpath, os.makedirs)
 try:
     readlink = wrap(mkpath, os.readlink, decorator=decode)
@@ -479,6 +516,7 @@ except AttributeError:
 
 realpath = wrap(mkpath, os.path.realpath, decorator=decode)
 relpath = wrap(mkpath, os.path.relpath, decorator=decode)
+remove = wrap(mkpath, os.remove)
 stat = wrap(mkpath, os.stat)
 unlink = wrap(mkpath, os.unlink)
 walk = wrap(mkpath, os.walk)
