@@ -94,7 +94,6 @@ class _BaseThread(QtCore.QThread):
     def _pending(self):
         return self._force_notify or self._file_paths or self._force_config
 
-    # pylint: disable=no-self-use
     def refresh(self):
         """Do any housekeeping necessary in response to repository changes."""
         return
@@ -258,12 +257,10 @@ if AVAILABLE == 'inotify':
             context = self.context
             try:
                 if self._worktree is not None:
-                    tracked_dirs = set(
-                        [
-                            os.path.dirname(os.path.join(self._worktree, path))
-                            for path in gitcmds.tracked_files(context)
-                        ]
-                    )
+                    tracked_dirs = {
+                        os.path.dirname(os.path.join(self._worktree, path))
+                        for path in gitcmds.tracked_files(context)
+                    }
                     self._refresh_watches(
                         tracked_dirs,
                         self._worktree_wd_to_path_map,
@@ -457,10 +454,10 @@ if AVAILABLE == 'pywin32':
                         timeout = self._NOTIFICATION_DELAY
                     else:
                         timeout = win32event.INFINITE
-                    rc = win32event.WaitForMultipleObjects(events, False, timeout)
+                    status = win32event.WaitForMultipleObjects(events, False, timeout)
                     if not self._running:
                         break
-                    if rc == win32event.WAIT_TIMEOUT:
+                    if status == win32event.WAIT_TIMEOUT:
                         self.notify()
                     else:
                         self._handle_results()
