@@ -5,6 +5,7 @@ import os
 from os.path import join
 import subprocess
 import threading
+import time
 
 from . import core
 from .compat import int_types
@@ -35,8 +36,8 @@ OID_LENGTH = 40
 _index_lock = threading.Lock()
 
 
-def dashify(s):
-    return s.replace('_', '-')
+def dashify(value):
+    return value.replace('_', '-')
 
 
 def is_git_dir(git_dir):
@@ -66,12 +67,12 @@ def is_git_dir(git_dir):
     return result
 
 
-def is_git_file(f):
-    return core.isfile(f) and os.path.basename(f) == '.git'
+def is_git_file(filename):
+    return core.isfile(filename) and os.path.basename(filename) == '.git'
 
 
-def is_git_worktree(d):
-    return is_git_dir(join(d, '.git'))
+def is_git_worktree(dirname):
+    return is_git_dir(join(dirname, '.git'))
 
 
 def is_git_repository(path):
@@ -187,7 +188,6 @@ class Git(object):
         self._valid = {}  #: Store the result of is_git_dir() for performance
         self.set_worktree(core.getcwd())
 
-    # pylint: disable=no-self-use
     def is_git_repository(self, path):
         return is_git_repository(path)
 
@@ -261,6 +261,7 @@ class Git(object):
         :param _cwd: working directory, defaults to the current directory.
         :param _decode: whether to decode output, defaults to True.
         :param _encoding: default encoding, defaults to None (utf-8).
+        :param _readonly: avoid taking the index lock. Assume the command is read-only.
         :param _raw: do not strip trailing whitespace.
         :param _stdin: optional stdin filehandle.
         :returns (status, out, err): exit status, stdout, stderr
@@ -279,10 +280,12 @@ class Git(object):
             # process from the console it should fork and call os.setsid().
             extra['preexec_fn'] = os.setsid
 
+        start_time = time.time()
+
         # Start the process
         # Guard against thread-unsafe .git/index.lock files
         if not _readonly:
-            _index_lock.acquire()
+            _index_lock.acquire()  # pylint: disable=consider-using-with
         try:
             status, out, err = core.run_command(
                 command,
@@ -299,22 +302,28 @@ class Git(object):
             if not _readonly:
                 _index_lock.release()
 
+        end_time = time.time()
+        elapsed_time = abs(end_time - start_time)
+
         if not _raw and out is not None:
             out = core.UStr(out.rstrip('\n'), out.encoding)
 
         cola_trace = GIT_COLA_TRACE
         if cola_trace == 'trace':
-            msg = 'trace: ' + core.list2cmdline(command)
+            msg = 'trace: %.3fs: %s' % (elapsed_time, core.list2cmdline(command))
             Interaction.log_status(status, msg, '')
         elif cola_trace == 'full':
             if out or err:
                 core.print_stderr(
-                    "%s -> %d: '%s' '%s'" % (' '.join(command), status, out, err)
+                    "# %.3fs: %s -> %d: '%s' '%s'"
+                    % (elapsed_time, ' '.join(command), status, out, err)
                 )
             else:
-                core.print_stderr("%s -> %d" % (' '.join(command), status))
+                core.print_stderr(
+                    '# %.3fs: %s -> %d' % (elapsed_time, ' '.join(command), status)
+                )
         elif cola_trace:
-            core.print_stderr(' '.join(command))
+            core.print_stderr('# %.3fs: %s' % (elapsed_time, ' '.join(command)))
 
         # Allow access to the command's status code
         return (status, out, err)
@@ -353,14 +362,14 @@ class Git(object):
         call.extend(args)
         try:
             result = self.execute(call, **_kwargs)
-        except OSError as e:
-            if WIN32 and e.errno == errno.ENOENT:
+        except OSError as exc:
+            if WIN32 and exc.errno == errno.ENOENT:
                 # see if git exists at all. on win32 it can fail with ENOENT in
                 # case of argv overflow. we should be safe from that but use
                 # defensive coding for the worst-case scenario. On UNIX
                 # we have ENAMETOOLONG but that doesn't exist on Windows.
                 if _git_is_installed():
-                    raise e
+                    raise exc
                 _print_win32_git_hint()
             result = (1, '', "error: unable to execute '%s'" % GIT)
         return result
@@ -398,20 +407,20 @@ def transform_kwargs(**kwargs):
     args = []
     types_to_stringify = (ustr, float, str) + int_types
 
-    for k, v in kwargs.items():
+    for k, value in kwargs.items():
         if len(k) == 1:
             dashes = '-'
-            eq = ''
+            equals = ''
         else:
             dashes = '--'
-            eq = '='
+            equals = '='
         # isinstance(False, int) is True, so we have to check bool first
-        if isinstance(v, bool):
-            if v:
+        if isinstance(value, bool):
+            if value:
                 args.append('%s%s' % (dashes, dashify(k)))
             # else: pass  # False is ignored; flag=False inhibits --flag
-        elif isinstance(v, types_to_stringify):
-            args.append('%s%s%s%s' % (dashes, dashify(k), eq, v))
+        elif isinstance(value, types_to_stringify):
+            args.append('%s%s%s%s' % (dashes, dashify(k), equals, value))
 
     return args
 
