@@ -22,8 +22,8 @@ def create(context):
 class MainModel(QtCore.QObject):
     """Repository status model"""
 
-    # TODO this class can probably be split apart into a DiffModel,
-    # CommitMessageModel, StatusModel, and an AppStatusStateMachine.
+    # Refactor: split this class apart into separate DiffModel, CommitMessageModel,
+    # StatusModel, and an DiffEditorState.
 
     # Signals
     about_to_update = Signal()
@@ -51,15 +51,18 @@ class MainModel(QtCore.QObject):
     mode_untracked_diff = 'untracked-diff'  # Diffing an untracked file
     mode_index = 'index'  # Comparing index to last commit
     mode_amend = 'amend'  # Amending a commit
+    mode_diff = 'diff'  # Diffing against an arbitrary commit
 
     # Modes where we can checkout files from the $head
-    modes_undoable = set((mode_amend, mode_index, mode_worktree))
+    modes_undoable = set((mode_amend, mode_diff, mode_index, mode_worktree))
 
     # Modes where we can partially stage files
-    modes_partially_stageable = set((mode_amend, mode_worktree, mode_untracked_diff))
+    modes_partially_stageable = set(
+        (mode_amend, mode_diff, mode_worktree, mode_untracked_diff)
+    )
 
     # Modes where we can partially unstage files
-    modes_unstageable = set((mode_amend, mode_index))
+    modes_unstageable = set((mode_amend, mode_diff, mode_index))
 
     unstaged = property(lambda self: self.modified + self.unmerged + self.untracked)
     """An aggregate of the modified, unmerged, and untracked file lists."""
@@ -82,8 +85,10 @@ class MainModel(QtCore.QObject):
         self.file_type = Types.TEXT
         self.mode = self.mode_none
         self.filename = None
+        self.is_cherry_picking = False
         self.is_merging = False
         self.is_rebasing = False
+        self.is_applying_patch = False
         self.currentbranch = ''
         self.directory = ''
         self.project = ''
@@ -113,23 +118,29 @@ class MainModel(QtCore.QObject):
         if cwd:
             self.set_worktree(cwd)
 
-    def unstageable(self):
+    def is_diff_mode(self):
+        """Are we in diff mode?"""
+        return self.mode == self.mode_diff
+
+    def is_unstageable(self):
+        """Are we in a mode that supports "unstage" actions?"""
         return self.mode in self.modes_unstageable
 
-    def amending(self):
+    def is_amend_mode(self):
+        """Are we amending a commit?"""
         return self.mode == self.mode_amend
 
-    def undoable(self):
-        """Whether we can checkout files from the $head."""
+    def is_undoable(self):
+        """Can we checkout from the current branch or head ref?"""
         return self.mode in self.modes_undoable
 
-    def partially_stageable(self):
+    def is_partially_stageable(self):
         """Whether partial staging should be allowed."""
         return self.mode in self.modes_partially_stageable
 
-    def stageable(self):
+    def is_stageable(self):
         """Whether staging should be allowed."""
-        return self.partially_stageable() or self.mode == self.modes_untracked
+        return self.is_partially_stageable() or self.mode == self.mode_untracked
 
     def all_branches(self):
         return self.local_branches + self.remote_branches
@@ -230,16 +241,29 @@ class MainModel(QtCore.QObject):
     def set_directory(self, path):
         self.directory = path
 
-    def set_mode(self, mode):
-        if self.amending():
-            if mode != self.mode_none:
-                return
-        if self.is_merging and mode == self.mode_amend:
+    def set_mode(self, mode, head=None):
+        """Set the current editing mode (worktree, index, amending, ...)"""
+        # Do not allow going into index or worktree mode when amending.
+        if self.is_amend_mode() and mode != self.mode_none:
+            return
+        # We cannot amend in the middle of git cherry-pick, git am or git merge.
+        if (
+            self.is_cherry_picking or self.is_merging or self.is_applying_patch
+        ) and mode == self.mode_amend:
             mode = self.mode
-        if mode == self.mode_amend:
-            head = 'HEAD^'
+
+        # Stay in diff mode until explicitly reset.
+        if self.mode == self.mode_diff and mode != self.mode_none:
+            mode = self.mode_diff
+            head = head or self.head
         else:
-            head = 'HEAD'
+            # If we are amending then we'll use "HEAD^", otherwise use the specified
+            # head or "HEAD" if head has not been specified.
+            if mode == self.mode_amend:
+                head = 'HEAD^'
+            elif not head:
+                head = 'HEAD'
+
         self.head = head
         self.mode = mode
         self.mode_changed.emit(mode)
@@ -258,9 +282,16 @@ class MainModel(QtCore.QObject):
         self.updated.emit()
 
     def update_file_status(self, update_index=False):
+        """Update modified/staged files status"""
         self.emit_about_to_update()
         self.update_files(update_index=update_index, emit=True)
 
+    def update_file_merge_status(self):
+        """Update modified/staged files and Merge/Rebase/Cherry-pick status"""
+        self.emit_about_to_update()
+        self._update_merge_rebase_status()
+        self.update_file_status()
+
     def update_status(self, update_index=False, reset=False):
         # Give observers a chance to respond
         self.emit_about_to_update()
@@ -344,20 +375,25 @@ class MainModel(QtCore.QObject):
         self.refs_updated.emit()
 
     def _update_merge_rebase_status(self):
+        cherry_pick_head = self.git.git_path('CHERRY_PICK_HEAD')
         merge_head = self.git.git_path('MERGE_HEAD')
         rebase_merge = self.git.git_path('rebase-merge')
+        rebase_apply = self.git.git_path('rebase-apply', 'applying')
+        self.is_cherry_picking = cherry_pick_head and core.exists(cherry_pick_head)
         self.is_merging = merge_head and core.exists(merge_head)
         self.is_rebasing = rebase_merge and core.exists(rebase_merge)
-        if self.is_merging and self.mode == self.mode_amend:
+        self.is_applying_patch = rebase_apply and core.exists(rebase_apply)
+        if self.mode == self.mode_amend and (
+            self.is_merging or self.is_cherry_picking or self.is_applying_patch
+        ):
             self.set_mode(self.mode_none)
 
     def _update_commitmsg(self):
         """Check for merge message files and update the commit message
 
-        The message is cleared when the merge completes
-
+        The message is cleared when the merge completes.
         """
-        if self.amending():
+        if self.is_amend_mode():
             return
         # Check if there's a message file in .git/
         context = self.context
@@ -431,21 +467,6 @@ class MainModel(QtCore.QObject):
         """
         return self.git.branch(name, base, track=track, force=force)
 
-    def cherry_pick_list(self, revs):
-        """Cherry-picks each revision into the current branch.
-        Returns a list of command output strings (1 per cherry pick)"""
-        if not revs:
-            return []
-        outs = []
-        errs = []
-        status = 0
-        for rev in revs:
-            stat, out, err = self.git.cherry_pick(rev)
-            status = max(stat, status)
-            outs.append(out)
-            errs.append(err)
-        return (status, '\n'.join(outs), '\n'.join(errs))
-
     def is_commit_published(self):
         """Return True if the latest commit exists in any remote branch"""
         return bool(self.git.branch(r=True, contains='HEAD')[STDOUT])
