@@ -14,6 +14,7 @@ AUTOCOMPLETE_PATHS = 'cola.autocompletepaths'
 AUTOTEMPLATE = 'cola.autoloadcommittemplate'
 BACKGROUND_EDITOR = 'cola.backgroundeditor'
 BLAME_VIEWER = 'cola.blameviewer'
+BLOCK_CURSOR = 'cola.blockcursor'
 BOLD_HEADERS = 'cola.boldheaders'
 CHECK_CONFLICTS = 'cola.checkconflicts'
 CHECK_PUBLISHED_COMMITS = 'cola.checkpublishedcommits'
@@ -22,6 +23,7 @@ DIFFCONTEXT = 'gui.diffcontext'
 DIFFTOOL = 'diff.tool'
 DISPLAY_UNTRACKED = 'gui.displayuntracked'
 EDITOR = 'gui.editor'
+ENABLE_GRAVATAR = 'cola.gravatar'
 EXPANDTAB = 'cola.expandtab'
 FONTDIFF = 'cola.fontdiff'
 HIDPI = 'cola.hidpi'
@@ -34,6 +36,7 @@ MERGE_KEEPBACKUP = 'merge.keepbackup'
 MERGE_SUMMARY = 'merge.summary'
 MERGE_VERBOSITY = 'merge.verbosity'
 MERGETOOL = 'merge.tool'
+MOUSE_ZOOM = 'cola.mousezoom'
 RESIZE_BROWSER_COLUMNS = 'cola.resizebrowsercolumns'
 SAFE_MODE = 'cola.safemode'
 SAVEWINDOWSETTINGS = 'cola.savewindowsettings'
@@ -55,6 +58,7 @@ class Defaults(object):
     # These should match Git's defaults for git-defined values.
     autotemplate = False
     blame_viewer = 'git gui blame'
+    block_cursor = True
     bold_headers = False
     check_conflicts = True
     check_published_commits = True
@@ -63,6 +67,7 @@ class Defaults(object):
     diff_context = 5
     difftool = 'xxdiff'
     editor = 'gvim'
+    enable_gravatar = True
     expandtab = False
     history_browser = 'gitk'
     icon_theme = 'default'
@@ -73,6 +78,7 @@ class Defaults(object):
     merge_keep_backup = True
     merge_summary = True
     merge_verbosity = 2
+    mouse_zoom = True
     resize_browser_columns = False
     save_window_settings = True
     safe_mode = False
@@ -94,6 +100,11 @@ def blame_viewer(context):
     return context.cfg.get(BLAME_VIEWER, default=default)
 
 
+def block_cursor(context):
+    """Should we display a block cursor in diff editors?"""
+    return context.cfg.get(BLOCK_CURSOR, default=Defaults.block_cursor)
+
+
 def bold_headers(context):
     """Should we bold the Status column headers?"""
     return context.cfg.get(BOLD_HEADERS, default=Defaults.bold_headers)
@@ -144,15 +155,18 @@ def fallback_editor():
     for env in editor_variables:
         env_editor = core.getenv(env)
         if env_editor:
-            return editor
+            return env_editor
 
     return Defaults.editor
 
 
 def _remap_editor(app):
-    """Remap a configured editorinto a visual editor name"""
-    # We do this for vim users because this configuration is convenient.
-    return {'vim': 'gvim -f'}.get(app, app)
+    """Remap a configured editor into a visual editor name"""
+    # We do this for vim users because this configuration is convenient for new users.
+    return {
+        'vim': 'gvim -f',
+        'nvim': 'nvim-qt --nofork',
+    }.get(app, app)
 
 
 def comment_char(context):
@@ -160,6 +174,11 @@ def comment_char(context):
     return context.cfg.get(COMMENT_CHAR, default=Defaults.comment_char)
 
 
+def enable_gravatar(context):
+    """Is gravatar enabled?"""
+    return context.cfg.get(ENABLE_GRAVATAR, default=Defaults.enable_gravatar)
+
+
 def default_history_browser():
     """Return the default history browser (e.g. git-dag, gitk)"""
     if utils.is_win32():
@@ -197,6 +216,11 @@ def maxrecent(context):
     return value
 
 
+def mouse_zoom(context):
+    """Should we zoom text when using Ctrl + MouseWheel scroll"""
+    return context.cfg.get(MOUSE_ZOOM, default=Defaults.mouse_zoom)
+
+
 def spellcheck(context):
     """Should we spellcheck commit messages?"""
     return context.cfg.get(SPELL_CHECK, default=Defaults.spellcheck)
@@ -244,7 +268,7 @@ class PreferencesModel(QtCore.QObject):
 
     def set_config(self, source, config, value):
         """Set a configuration value"""
-        if source == 'repo':
+        if source == 'local':
             self.config.set_repo(config, value)
         else:
             self.config.set_user(config, value)
@@ -252,7 +276,7 @@ class PreferencesModel(QtCore.QObject):
 
     def get_config(self, source, config):
         """Get a configured value"""
-        if source == 'repo':
+        if source == 'local':
             value = self.config.get_repo(config)
         else:
             value = self.config.get(config)
