@@ -29,31 +29,38 @@ def active_window():
     return QtWidgets.QApplication.activeWindow()
 
 
-def connect_action(action, fn):
+def current_palette():
+    """Return the QPalette for the current application"""
+    return QtWidgets.QApplication.instance().palette()
+
+
+def connect_action(action, func):
     """Connect an action to a function"""
-    action.triggered[bool].connect(lambda x: fn())
+    action.triggered[bool].connect(lambda x: func(), type=Qt.QueuedConnection)
 
 
-def connect_action_bool(action, fn):
+def connect_action_bool(action, func):
     """Connect a triggered(bool) action to a function"""
-    action.triggered[bool].connect(fn)
+    action.triggered[bool].connect(func, type=Qt.QueuedConnection)
 
 
-def connect_button(button, fn):
+def connect_button(button, func):
     """Connect a button to a function"""
     # Some versions of Qt send the `bool` argument to the clicked callback,
     # and some do not.  The lambda consumes all callback-provided arguments.
-    button.clicked.connect(lambda *args, **kwargs: fn())
+    button.clicked.connect(lambda *args, **kwargs: func(), type=Qt.QueuedConnection)
 
 
-def connect_checkbox(widget, fn):
+def connect_checkbox(widget, func):
     """Connect a checkbox to a function taking bool"""
-    widget.clicked.connect(lambda *args, **kwargs: fn(get(checkbox)))
+    widget.clicked.connect(
+        lambda *args, **kwargs: func(get(checkbox)), type=Qt.QueuedConnection
+    )
 
 
-def connect_released(button, fn):
+def connect_released(button, func):
     """Connect a button to a function"""
-    button.released.connect(fn)
+    button.released.connect(func, type=Qt.QueuedConnection)
 
 
 def button_action(button, action):
@@ -61,9 +68,9 @@ def button_action(button, action):
     connect_button(button, action.trigger)
 
 
-def connect_toggle(toggle, fn):
+def connect_toggle(toggle, func):
     """Connect a toggle button to a function"""
-    toggle.toggled.connect(fn)
+    toggle.toggled.connect(func, type=Qt.QueuedConnection)
 
 
 def disconnect(signal):
@@ -74,7 +81,7 @@ def disconnect(signal):
         pass
 
 
-def get(widget):
+def get(widget, default=None):
     """Query a widget for its python value"""
     if hasattr(widget, 'isChecked'):
         value = widget.isChecked()
@@ -89,7 +96,7 @@ def get(widget):
     elif hasattr(widget, 'date'):
         value = widget.date().toString(Qt.ISODate)
     else:
-        value = None
+        value = default
     return value
 
 
@@ -304,9 +311,7 @@ def prompt_n(msg, inputs):
             long_value = k + v
 
     metrics = QtGui.QFontMetrics(dialog.font())
-    min_width = metrics.width(long_value) + 100
-    if min_width > 720:
-        min_width = 720
+    min_width = min(720, metrics.width(long_value) + 100)
     dialog.setMinimumWidth(min_width)
 
     ok_b = ok_button(msg, enabled=False)
@@ -321,7 +326,9 @@ def prompt_n(msg, inputs):
         lineedit = QtWidgets.QLineEdit()
         # Enable the OK button only when all fields have been populated
         # pylint: disable=no-member
-        lineedit.textChanged.connect(lambda x: ok_b.setEnabled(all(get_values())))
+        lineedit.textChanged.connect(
+            lambda x: ok_b.setEnabled(all(get_values())), type=Qt.QueuedConnection
+        )
         if value:
             lineedit.setText(value)
         form_widgets.append((name, lineedit))
@@ -338,13 +345,36 @@ def prompt_n(msg, inputs):
 
     accepted = dialog.exec_() == QtWidgets.QDialog.Accepted
     text = get_values()
-    ok = accepted and all(text)
-    return (ok, text)
+    success = accepted and all(text)
+    return (success, text)
+
+
+def standard_item_type_value(value):
+    """Return a custom UserType for use in QTreeWidgetItem.type() overrides"""
+    return custom_item_type_value(QtGui.QStandardItem, value)
+
+
+def graphics_item_type_value(value):
+    """Return a custom UserType for use in QGraphicsItem.type() overrides"""
+    return custom_item_type_value(QtWidgets.QGraphicsItem, value)
+
+
+def custom_item_type_value(cls, value):
+    """Return a custom cls.UserType for use in cls.type() overrides"""
+    user_type = enum_value(cls.UserType)
+    return user_type + value
+
+
+def enum_value(value):
+    """Qt6 has enums with an inner '.value' attribute."""
+    if hasattr(value, 'value'):
+        value = value.value
+    return value
 
 
 class TreeWidgetItem(QtWidgets.QTreeWidgetItem):
 
-    TYPE = QtGui.QStandardItem.UserType + 101
+    TYPE = standard_item_type_value(101)
 
     def __init__(self, path, icon, deleted):
         QtWidgets.QTreeWidgetItem.__init__(self)
@@ -363,7 +393,7 @@ def paths_from_indexes(model, indexes, item_type=TreeWidgetItem.TYPE, item_filte
     return paths_from_items(items, item_type=item_type, item_filter=item_filter)
 
 
-def _true_filter(_x):
+def _true_filter(_value):
     return True
 
 
@@ -498,30 +528,46 @@ def persist_clipboard():
     QtWidgets.QApplication.sendEvent(clipboard, event)
 
 
-def add_action_bool(widget, text, fn, checked, *shortcuts):
+def add_action_bool(widget, text, func, checked, *shortcuts):
     tip = text
-    action = _add_action(widget, text, tip, fn, connect_action_bool, *shortcuts)
+    action = _add_action(widget, text, tip, func, connect_action_bool, *shortcuts)
     action.setCheckable(True)
     action.setChecked(checked)
     return action
 
 
-def add_action(widget, text, fn, *shortcuts):
+def add_action(widget, text, func, *shortcuts):
+    """Create a QAction and bind it to the `func` callback and hotkeys"""
     tip = text
-    return _add_action(widget, text, tip, fn, connect_action, *shortcuts)
+    return _add_action(widget, text, tip, func, connect_action, *shortcuts)
 
 
-def add_action_with_status_tip(widget, text, tip, fn, *shortcuts):
-    return _add_action(widget, text, tip, fn, connect_action, *shortcuts)
+def add_action_with_icon(widget, icon, text, func, *shortcuts):
+    """Create a QAction using a custom icon bound to the `func` callback and hotkeys"""
+    tip = text
+    action = _add_action(widget, text, tip, func, connect_action, *shortcuts)
+    action.setIcon(icon)
+    return action
 
 
-def _add_action(widget, text, tip, fn, connect, *shortcuts):
+def add_action_with_status_tip(widget, text, tip, func, *shortcuts):
+    return _add_action(widget, text, tip, func, connect_action, *shortcuts)
+
+
+def menu_separator(widget):
+    """Return a QAction whose isSeparator() returns true. Used in context menus"""
+    action = QtWidgets.QAction('', widget)
+    action.setSeparator(True)
+    return action
+
+
+def _add_action(widget, text, tip, func, connect, *shortcuts):
     action = QtWidgets.QAction(text, widget)
     if hasattr(action, 'setIconVisibleInMenu'):
         action.setIconVisibleInMenu(True)
     if tip:
         action.setStatusTip(tip)
-    connect(action, fn)
+    connect(action, func)
     if shortcuts:
         action.setShortcuts(shortcuts)
         if hasattr(Qt, 'WidgetWithChildrenShortcut'):
@@ -588,9 +634,9 @@ def desktop_size():
 def center_on_screen(widget):
     """Move widget to the center of the default screen"""
     width, height = desktop_size()
-    cx = width // 2
-    cy = height // 2
-    widget.move(cx - widget.width() // 2, cy - widget.height() // 2)
+    center_x = width // 2
+    center_y = height // 2
+    widget.move(center_x - widget.width() // 2, center_y - widget.height() // 2)
 
 
 def default_size(parent, width, height, use_parent_height=True):
@@ -684,7 +730,7 @@ def tool_button():
     return button
 
 
-def create_action_button(tooltip=None, icon=None):
+def create_action_button(tooltip=None, icon=None, visible=True):
     """Create a small toolbutton for use in dock title widgets"""
     button = tool_button()
     if tooltip is not None:
@@ -692,6 +738,7 @@ def create_action_button(tooltip=None, icon=None):
     if icon is not None:
         button.setIcon(icon)
         button.setIconSize(QtCore.QSize(defs.small_icon, defs.small_icon))
+    button.setVisible(visible)
     return button
 
 
@@ -801,7 +848,7 @@ class DockTitleBarWidget(QtWidgets.QFrame):
         self.toggle_button.setToolTip(tooltip)
 
 
-def create_dock(name, title, parent, stretch=True, widget=None, fn=None):
+def create_dock(name, title, parent, stretch=True, widget=None, func=None):
     """Create a dock widget and set it up accordingly."""
     dock = QtWidgets.QDockWidget(parent)
     dock.setWindowTitle(title)
@@ -811,9 +858,8 @@ def create_dock(name, title, parent, stretch=True, widget=None, fn=None):
     dock.setAutoFillBackground(True)
     if hasattr(parent, 'dockwidgets'):
         parent.dockwidgets.append(dock)
-    if fn:
-        widget = fn(dock)
-        assert isinstance(widget, QtWidgets.QFrame), "Docked widget has to be a QFrame"
+    if func:
+        widget = func(dock)
     if widget:
         dock.setWidget(widget)
     return dock
@@ -878,36 +924,60 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     return button
 
 
+def create_toolbutton_with_callback(callback, text, icon, tooltip, layout=None):
+    """Create a toolbutton that runs the specified callback"""
+    toolbutton = create_toolbutton(text=text, layout=layout, tooltip=tooltip, icon=icon)
+    connect_button(toolbutton, callback)
+    return toolbutton
+
+
 # pylint: disable=line-too-long
-def mimedata_from_paths(context, paths):
+def mimedata_from_paths(context, paths, include_urls=True):
     """Return mimedata with a list of absolute path URLs
 
-    The text/x-moz-list format is always included by Qt, and doing
-    mimedata.removeFormat('text/x-moz-url') has no effect.
-    C.f. http://www.qtcentre.org/threads/44643-Dragging-text-uri-list-Qt-inserts-garbage
+    Set `include_urls` to False to prevent URLs from being included
+    in the mimedata. This is useful in some terminals that do not gracefully handle
+    multiple URLs being included in the payload.
 
-    gnome-terminal expects utf-16 encoded text, but other terminals,
-    e.g. terminator, prefer utf-8, so allow cola.dragencoding
-    to override the default.
+    This allows the mimedata to contain just plain a plain text value that we
+    are able to format ourselves.
 
+    Older verisons of gnome-terminal expected a utf-16 encoding, but that
+    behavior is no longer needed.
     """  # noqa
-    cfg = context.cfg
     abspaths = [core.abspath(path) for path in paths]
-    urls = [QtCore.QUrl.fromLocalFile(path) for path in abspaths]
-
-    mimedata = QtCore.QMimeData()
-    mimedata.setUrls(urls)
-
     paths_text = core.list2cmdline(abspaths)
-    encoding = cfg.get('cola.dragencoding', 'utf-16')
-    moz_text = core.encode(paths_text, encoding=encoding)
-    mimedata.setData('text/x-moz-url', moz_text)
+
+    # The text/x-moz-list format is always included by Qt, and doing
+    # mimedata.removeFormat('text/x-moz-url') has no effect.
+    # http://www.qtcentre.org/threads/44643-Dragging-text-uri-list-Qt-inserts-garbage
+    #
+    # Older versions of gnome-terminal expect utf-16 encoded text, but other terminals,
+    # e.g. terminator, expect utf-8, so use cola.dragencoding to override the default.
+    # NOTE: text/x-moz-url does not seem to be used/needed by modern versions of
+    # gnome-terminal, kitty, and terminator.
+    mimedata = QtCore.QMimeData()
+    mimedata.setText(paths_text)
+    if include_urls:
+        urls = [QtCore.QUrl.fromLocalFile(path) for path in abspaths]
+        encoding = context.cfg.get('cola.dragencoding', 'utf-16')
+        encoded_text = core.encode(paths_text, encoding=encoding)
+        mimedata.setUrls(urls)
+        mimedata.setData('text/x-moz-url', encoded_text)
 
     return mimedata
 
 
-def path_mimetypes():
-    return ['text/uri-list', 'text/x-moz-url']
+def path_mimetypes(include_urls=True):
+    """Return a list of mimetypes that we generate"""
+    mime_types = [
+        'text/plain',
+        'text/plain;charset=utf-8',
+    ]
+    if include_urls:
+        mime_types.append('text/uri-list')
+        mime_types.append('text/x-moz-url')
+    return mime_types
 
 
 class BlockSignals(object):
@@ -934,18 +1004,15 @@ class Channel(QtCore.QObject):
 
 
 class Task(QtCore.QRunnable):
-    """Disable auto-deletion to avoid gc issues
-
-    Python's garbage collector will try to double-free the task
-    once it's finished, so disable Qt's auto-deletion as a workaround.
-
-    """
+    """Run a task in the background and return the result using a Channel"""
 
     def __init__(self):
         QtCore.QRunnable.__init__(self)
 
         self.channel = Channel()
         self.result = None
+        # Python's garbage collector will try to double-free the task
+        # once it's finished, so disable Qt's auto-deletion as a workaround.
         self.setAutoDelete(False)
 
     def run(self):
@@ -953,9 +1020,9 @@ class Task(QtCore.QRunnable):
         self.channel.result.emit(self.result)
         self.channel.finished.emit(self)
 
-    # pylint: disable=no-self-use
     def task(self):
-        return None
+        """Perform a long-running task"""
+        return ()
 
     def connect(self, handler):
         self.channel.result.connect(handler, type=Qt.QueuedConnection)
@@ -964,15 +1031,15 @@ class Task(QtCore.QRunnable):
 class SimpleTask(Task):
     """Run a simple callable as a task"""
 
-    def __init__(self, fn, *args, **kwargs):
+    def __init__(self, func, *args, **kwargs):
         Task.__init__(self)
 
-        self.fn = fn
+        self.func = func
         self.args = args
         self.kwargs = kwargs
 
     def task(self):
-        return self.fn(*self.args, **self.kwargs)
+        return self.func(*self.args, **self.kwargs)
 
 
 class RunTask(QtCore.QObject):
@@ -983,13 +1050,16 @@ class RunTask(QtCore.QObject):
         self.tasks = []
         self.task_details = {}
         self.threadpool = QtCore.QThreadPool.globalInstance()
-        self.result_fn = None
+        self.result_func = None
 
     def start(self, task, progress=None, finish=None, result=None):
         """Start the task and register a callback"""
-        self.result_fn = result
+        self.result_func = result
         if progress is not None:
             progress.show()
+            if hasattr(progress, 'start'):
+                progress.start()
+
         # prevents garbage collection bugs in certain PyQt4 versions
         self.tasks.append(task)
         task_id = id(task)
@@ -998,6 +1068,7 @@ class RunTask(QtCore.QObject):
         self.threadpool.start(task)
 
     def finish(self, task):
+        """The task has finished. Run the finish and result callbacks"""
         task_id = id(task)
         try:
             self.tasks.remove(task)
@@ -1010,6 +1081,8 @@ class RunTask(QtCore.QObject):
             finish = progress = result = None
 
         if progress is not None:
+            if hasattr(progress, 'stop'):
+                progress.stop()
             progress.hide()
 
         if result is not None:
@@ -1022,25 +1095,28 @@ class RunTask(QtCore.QObject):
 # Syntax highlighting
 
 
-def rgb(r, g, b):
+def rgb(red, green, blue):
+    """Create a QColor from r, g, b arguments"""
     color = QtGui.QColor()
-    color.setRgb(r, g, b)
+    color.setRgb(red, green, blue)
     return color
 
 
-def rgba(r, g, b, a=255):
-    color = rgb(r, g, b)
-    color.setAlpha(a)
+def rgba(red, green, blue, alpha=255):
+    """Create a QColor with alpha from r, g, b, a arguments"""
+    color = rgb(red, green, blue)
+    color.setAlpha(alpha)
     return color
 
 
-def RGB(args):
+def rgb_triple(args):
+    """Create a QColor from an argument with an [r, g, b] triple"""
     return rgb(*args)
 
 
 def rgb_css(color):
-    """Convert a QColor into an rgb(int, int, int) CSS string"""
-    return 'rgb(%d, %d, %d)' % (color.red(), color.green(), color.blue())
+    """Convert a QColor into an rgb #abcdef CSS string"""
+    return '#%s' % rgb_hex(color)
 
 
 def rgb_hex(color):
@@ -1048,22 +1124,51 @@ def rgb_hex(color):
     return '%02x%02x%02x' % (color.red(), color.green(), color.blue())
 
 
-def hsl(h, s, light):
+def clamp_color(value):
+    """Clamp an integer value between 0 and 255"""
+    return min(255, max(value, 0))
+
+
+def css_color(value):
+    """Convert a #abcdef hex string into a QColor"""
+    if value.startswith('#'):
+        value = value[1:]
+    try:
+        red = clamp_color(int(value[:2], base=16))  # ab
+    except ValueError:
+        red = 255
+    try:
+        green = clamp_color(int(value[2:4], base=16))  # cd
+    except ValueError:
+        green = 255
+    try:
+        blue = clamp_color(int(value[4:6], base=16))  # ef
+    except ValueError:
+        blue = 255
+    return rgb(red, green, blue)
+
+
+def hsl(hue, saturation, lightness):
+    """Return a QColor from an hue, saturation and lightness"""
     return QtGui.QColor.fromHslF(
-        utils.clamp(h, 0.0, 1.0), utils.clamp(s, 0.0, 1.0), utils.clamp(light, 0.0, 1.0)
+        utils.clamp(hue, 0.0, 1.0),
+        utils.clamp(saturation, 0.0, 1.0),
+        utils.clamp(lightness, 0.0, 1.0),
     )
 
 
-def hsl_css(h, s, light):
-    return rgb_css(hsl(h, s, light))
+def hsl_css(hue, saturation, lightness):
+    """Convert HSL values to a CSS #abcdef color string"""
+    return rgb_css(hsl(hue, saturation, lightness))
 
 
-def make_format(fg=None, bg=None, bold=False):
+def make_format(foreground=None, background=None, bold=False):
+    """Create a QTextFormat from the provided foreground, background and bold values"""
     fmt = QtGui.QTextCharFormat()
-    if fg:
-        fmt.setForeground(fg)
-    if bg:
-        fmt.setBackground(bg)
+    if foreground:
+        fmt.setForeground(foreground)
+    if background:
+        fmt.setBackground(background)
     if bold:
         fmt.setFontWeight(QtGui.QFont.Bold)
     return fmt
@@ -1076,7 +1181,7 @@ class ImageFormats(object):
         # portability: python3 data() returns bytes, python2 returns str
         decode = core.decode
         formats = [decode(x.data()) for x in formats_qba]
-        self.extensions = set(['.' + fmt for fmt in formats])
+        self.extensions = {'.' + fmt for fmt in formats}
 
     def ok(self, filename):
         _, ext = os.path.splitext(filename)
@@ -1138,3 +1243,18 @@ def get_selected_items(widget, idx):
     """Return the selected items under the top-level item"""
     item = widget.topLevelItem(idx)
     return tree_selection_items(item)
+
+
+def add_menu_actions(menu, menu_actions):
+    """Add actions to a menu, treating None as a separator"""
+    current_actions = menu.actions()
+    if current_actions:
+        first_action = current_actions[0]
+    else:
+        first_action = None
+        menu.addSeparator()
+
+    for action in menu_actions:
+        if action is None:
+            action = menu_separator(menu)
+        menu.insertAction(first_action, action)
