@@ -2,7 +2,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 from functools import partial
 
-from qtpy import QtCore
 from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
@@ -22,16 +21,17 @@ from .. import qtutils
 from .text import LineEdit
 
 
-def defer_fn(parent, title, fn, *args, **kwargs):
-    return qtutils.add_action(parent, title, partial(fn, *args, **kwargs))
+def defer_func(parent, title, func, *args, **kwargs):
+    """Return a QAction bound against a partial func with arguments"""
+    return qtutils.add_action(parent, title, partial(func, *args, **kwargs))
 
 
-def add_branch_to_menu(menu, branch, remote_branch, remote, upstream, fn):
+def add_branch_to_menu(menu, branch, remote_branch, remote, upstream, func):
     """Add a remote branch to the context menu"""
     branch_remote, _ = gitcmds.parse_remote_branch(remote_branch)
     if branch_remote != remote:
         menu.addSeparator()
-    action = defer_fn(menu, remote_branch, fn, branch, remote_branch)
+    action = defer_func(menu, remote_branch, func, branch, remote_branch)
     if remote_branch == upstream:
         action.setIcon(icons.star())
     menu.addAction(action)
@@ -41,12 +41,13 @@ def add_branch_to_menu(menu, branch, remote_branch, remote, upstream, fn):
 class AsyncGitActionTask(qtutils.Task):
     """Run git action asynchronously"""
 
-    def __init__(self, git_helper, action, args, kwarg):
+    def __init__(self, git_helper, action, args, kwarg, update_refs):
         qtutils.Task.__init__(self)
         self.git_helper = git_helper
         self.action = action
         self.args = args
         self.kwarg = kwarg
+        self.update_refs = update_refs
 
     def task(self):
         """Runs action and captures the result"""
@@ -112,7 +113,6 @@ class BranchesWidget(QtWidgets.QFrame):
     def refresh(self):
         icon = self.order_icon(self.model.ref_sort)
         self.sort_order_button.setIcon(icon)
-        self.tree.refresh()
 
 
 # pylint: disable=too-many-ancestors
@@ -131,18 +131,14 @@ class BranchesTreeWidget(standard.TreeWidget):
         self.setExpandsOnDoubleClick(False)
 
         self.current_branch = None
-        self.tree_helper = BranchesTreeHelper()
+        self.tree_helper = BranchesTreeHelper(self)
         self.git_helper = GitHelper(context)
         self.runtask = qtutils.RunTask(parent=self)
 
-        self._active = False
+        self._visible = False
+        self._needs_refresh = False
         self._tree_states = None
 
-        # The refresh timer is used to debounce update events.
-        # This makes it so that two events in quick succession only refresh once.
-        self._refresh_timer = QtCore.QTimer(self)
-        self._refresh_timer.timeout.connect(self._refresh)
-
         self.updated.connect(self.refresh, type=Qt.QueuedConnection)
         context.model.updated.connect(self.updated)
 
@@ -154,16 +150,14 @@ class BranchesTreeWidget(standard.TreeWidget):
         self.doubleClicked.connect(self.checkout_action)
 
     def refresh(self):
-        """Start the refresh timer"""
-        if self._refresh_timer.isActive():
-            self._refresh_timer.stop()
-        self._refresh_timer.start(120)
+        """Refresh the UI widgets to match the current state"""
+        self._needs_refresh = True
+        self._refresh()
 
     def _refresh(self):
         """Refresh the UI to match the updated state"""
-        self._refresh_timer.stop()
         # There is no need to refresh the UI when this widget is inactive.
-        if not self._active:
+        if not self._visible:
             return
         model = self.context.model
         self.current_branch = model.currentbranch
@@ -187,18 +181,26 @@ class BranchesTreeWidget(standard.TreeWidget):
 
         self.clear()
         self.addTopLevelItems([local, remote, tags])
+
+        if self._tree_states:
+            self._load_tree_state(self._tree_states)
+            self._tree_states = None
+
         self._update_branches()
 
     def showEvent(self, event):
         """Defer updating widgets until the widget is visible"""
-        if not self._active:
-            self._active = True
-            self.refresh()
+        if not self._visible:
+            self._visible = True
+            if self._needs_refresh:
+                self.refresh()
         return super(BranchesTreeWidget, self).showEvent(event)
 
     def _toggle_expanded(self, index):
         """Toggle expanded/collapsed state when items are clicked"""
-        self.setExpanded(index, not self.isExpanded(index))
+        item = self.itemFromIndex(index)
+        if item and item.childCount():
+            self.setExpanded(index, not self.isExpanded(index))
 
     def contextMenuEvent(self, event):
         """Build and execute the context menu"""
@@ -363,14 +365,19 @@ class BranchesTreeWidget(standard.TreeWidget):
         states = {}
         for item in self.items():
             states.update(self.tree_helper.save_state(item))
-
         return states
 
     def _load_tree_state(self, states):
         """Restore expanded items after rebuilding UI widgets"""
+        # Disable animations to eliminate redraw flicker.
+        animated = self.isAnimated()
+        self.setAnimated(False)
+
         for item in self.items():
-            if item.name in states:
-                self.tree_helper.load_state(item, states[item.name])
+            self.tree_helper.load_state(item, states.get(item.name, {}))
+        self.tree_helper.set_current_item()
+
+        self.setAnimated(animated)
 
     def _update_branches(self):
         """Query branch details using a background task"""
@@ -406,14 +413,10 @@ class BranchesTreeWidget(standard.TreeWidget):
             if status_str:
                 item.setText(0, '%s\t%s' % (item.text(0), status_str))
 
-        if self._tree_states:
-            self._load_tree_state(self._tree_states)
-            self._tree_states = None
-
-    def git_action_async(self, action, args, kwarg=None):
+    def git_action_async(self, action, args, kwarg=None, update_refs=False):
         if kwarg is None:
             kwarg = {}
-        task = AsyncGitActionTask(self.git_helper, action, args, kwarg)
+        task = AsyncGitActionTask(self.git_helper, action, args, kwarg, update_refs)
         progress = standard.progress(
             N_('Executing action %s') % action, N_('Updating'), self
         )
@@ -422,7 +425,8 @@ class BranchesTreeWidget(standard.TreeWidget):
     def git_action_completed(self, task):
         status, out, err = task.result
         self.git_helper.show_result(task.action, status, out, err)
-        self.context.model.update_refs()
+        if task.update_refs:
+            self.context.model.update_refs()
 
     def push_action(self):
         context = self.context
@@ -433,7 +437,7 @@ class BranchesTreeWidget(standard.TreeWidget):
             if remote and branch_name:
                 # we assume that user wants to "Push" the selected local
                 # branch to a remote with same name
-                self.git_action_async('push', [remote, branch_name])
+                self.git_action_async('push', [remote, branch_name], update_refs=True)
 
     def rename_action(self):
         branch = self.selected_refname()
@@ -441,7 +445,7 @@ class BranchesTreeWidget(standard.TreeWidget):
             N_('Enter New Branch Name'), title=N_('Rename branch'), text=branch
         )
         if ok and new_branch:
-            self.git_action_async('rename', [branch, new_branch])
+            self.git_action_async('rename', [branch, new_branch], update_refs=True)
 
     def pull_action(self):
         context = self.context
@@ -452,7 +456,7 @@ class BranchesTreeWidget(standard.TreeWidget):
         if remote_branch:
             remote, branch_name = gitcmds.parse_remote_branch(remote_branch)
             if remote and branch_name:
-                self.git_action_async('pull', [remote, branch_name])
+                self.git_action_async('pull', [remote, branch_name], update_refs=True)
 
     def delete_action(self):
         branch = self.selected_refname()
@@ -461,6 +465,8 @@ class BranchesTreeWidget(standard.TreeWidget):
 
         remote = False
         root = get_toplevel_item(self.selected_item())
+        if not root:
+            return
         if root.name == N_('Remote'):
             remote = True
 
@@ -479,13 +485,15 @@ class BranchesTreeWidget(standard.TreeWidget):
     def checkout_action(self):
         branch = self.selected_refname()
         if branch and branch != self.current_branch:
-            self.git_action_async('checkout', [branch])
+            self.git_action_async('checkout', [branch], update_refs=True)
 
     def checkout_new_branch_action(self):
         branch = self.selected_refname()
         if branch and branch != self.current_branch:
             _, new_branch = gitcmds.parse_remote_branch(branch)
-            self.git_action_async('checkout', ['-b', new_branch, branch])
+            self.git_action_async(
+                'checkout', ['-b', new_branch, branch], update_refs=True
+            )
 
     def selected_refname(self):
         return getattr(self.selected_item(), 'refname', None)
@@ -533,14 +541,6 @@ class BranchTreeWidgetItem(QtWidgets.QTreeWidgetItem):
             self.setIcon(0, icon)
         self.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
 
-    # TODO: review standard.py 317.
-    # original function returns 'QTreeWidgetItem' object which has no
-    # attribute 'rowCount'. This workaround fix error throw when
-    # navigating with keyboard and press left key
-    @staticmethod
-    def rowCount():
-        return 1
-
 
 class TreeEntry(object):
     """Tree representation for the branches widget
@@ -666,7 +666,8 @@ def expand_item_parents(item):
     """Expand tree parents from item"""
     parent = item.parent()
     while parent is not None:
-        parent.setExpanded(True)
+        if not parent.isExpanded():
+            parent.setExpanded(True)
         parent = parent.parent()
 
 
@@ -699,24 +700,49 @@ def get_toplevel_item(item):
 
 
 class BranchesTreeHelper(object):
+    """Save and restore the tree state"""
+
+    def __init__(self, tree):
+        self.tree = tree
+        self.current_item = None
+
+    def set_current_item(self):
+        """Reset the current item"""
+        if self.current_item is not None:
+            self.tree.setCurrentItem(self.current_item)
+        self.current_item = None
+
     def load_state(self, item, state):
         """Load expanded items from a dict"""
-        if state.keys():
+        if not state:
+            return
+        if state.get('expanded', False) and not item.isExpanded():
             item.setExpanded(True)
+        if state.get('selected', False) and not item.isSelected():
+            item.setSelected(True)
+            self.current_item = item
 
+        children_state = state.get('children', {})
+        if not children_state:
+            return
         for i in range(item.childCount()):
             child = item.child(i)
-            if child.name in state:
-                self.load_state(child, state[child.name])
+            self.load_state(child, children_state.get(child.name, {}))
 
     def save_state(self, item):
-        """Save expanded items in a dict"""
-        result = {item.name: {}}
-
-        if item.isExpanded():
-            for i in range(item.childCount()):
-                child = item.child(i)
-                result[item.name].update(self.save_state(child))
+        """Save the selected and expanded item state into a dict"""
+        expanded = item.isExpanded()
+        selected = item.isSelected()
+        children = {}
+        entry = {
+            'children': children,
+            'expanded': expanded,
+            'selected': selected,
+        }
+        result = {item.name: entry}
+        for i in range(item.childCount()):
+            child = item.child(i)
+            children.update(self.save_state(child))
 
         return result
 
