@@ -1,4 +1,4 @@
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import operator
 
 from qtpy import QtCore
@@ -196,7 +196,7 @@ class RemoteEditor(standard.Dialog):
             self.current_name = name
 
             self.refresh(select=False)
-            remotes = utils.seq(self.remote_list)
+            remotes = utils.Sequence(self.remote_list)
             idx = remotes.index(name)
             self.select_remote(idx)
             gather = False  # already done by select_remote()
@@ -237,23 +237,23 @@ class RemoteEditor(standard.Dialog):
     def refresh(self, select=True):
         git = self.context.git
         remotes = git.remote()[STDOUT].splitlines()
-        self.remotes.blockSignals(True)  # ignore selection change signals
-        self.remotes.clear()
-        self.remotes.addItems(remotes)
-        self.remote_list = remotes
+        # Ignore notifications from self.remotes while mutating.
+        with qtutils.BlockSignals(self.remotes):
+            self.remotes.clear()
+            self.remotes.addItems(remotes)
+            self.remote_list = remotes
 
-        for idx in range(len(remotes)):
-            item = self.remotes.item(idx)
-            item.setFlags(item.flags() | Qt.ItemIsEditable)
+            for idx in range(len(remotes)):
+                item = self.remotes.item(idx)
+                item.setFlags(item.flags() | Qt.ItemIsEditable)
 
-        self.remotes.blockSignals(False)
         if select:
             if not self.current_name and remotes:
                 # Nothing is selected; select the first item
                 self.select_remote(0)
             elif self.current_name and remotes:
                 # Reselect the previously selected item
-                remote_seq = utils.seq(remotes)
+                remote_seq = utils.Sequence(remotes)
                 idx = remote_seq.index(self.current_name)
                 if idx >= 0:
                     item = self.remotes.item(idx)
@@ -434,7 +434,7 @@ class RemoteWidget(QtWidgets.QWidget):
         self.remote_name = lineedit(context, N_('Name for the new remote'))
         self.remote_url = lineedit(context, 'git://git.example.com/repo.git')
         self.open_button = qtutils.create_button(
-            text=N_('Open...'), icon=icons.folder(), tooltip=N_('Select repository')
+            text=N_('Browse...'), icon=icons.folder(), tooltip=N_('Select repository')
         )
 
         self.url_layout = qtutils.hbox(
@@ -470,6 +470,6 @@ class RemoteWidget(QtWidgets.QWidget):
 
     def open_repo(self):
         git = self.context.git
-        repo = qtutils.opendir_dialog(N_('Open Git Repository...'), core.getcwd())
+        repo = qtutils.opendir_dialog(N_('Open Git Repository'), core.getcwd())
         if repo and git.is_git_repository(repo):
             self.url = repo
