@@ -9,28 +9,23 @@ from .. import hotkeys
 from .. import qtutils
 from ..i18n import N_
 from .standard import TreeWidget
-from .diff import COMMITS_SELECTED
-from .diff import FILES_SELECTED
-
-HISTORIES_SELECTED = 'HISTORIES_SELECTED'
-DIFFTOOL_SELECTED = 'DIFFTOOL_SELECTED'
 
 
 # pylint: disable=too-many-ancestors
 class FileWidget(TreeWidget):
 
+    files_selected = Signal(object)
+    difftool_selected = Signal(object)
+    histories_selected = Signal(object)
     grab_file = Signal(object)
 
-    def __init__(self, context, notifier, parent):
+    def __init__(self, context, parent):
         TreeWidget.__init__(self, parent)
         self.context = context
-        self.notifier = notifier
 
         labels = [N_('Filename'), N_('Additions'), N_('Deletions')]
         self.setHeaderLabels(labels)
 
-        notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
-
         self.show_history_action = qtutils.add_action(
             self, N_('Show History'), self.show_history, hotkeys.HISTORY
         )
@@ -52,23 +47,35 @@ class FileWidget(TreeWidget):
 
     def selection_changed(self):
         items = self.selected_items()
-        self.notifier.notify_observers(FILES_SELECTED, [i.path for i in items])
+        self.files_selected.emit([i.path for i in items])
 
     def commits_selected(self, commits):
         if not commits:
+            self.clear()
             return
+
         git = self.context.git
-        commit = commits[0]
-        oid = commit.oid
-        status, out, _ = git.show(
-            oid, z=True, numstat=True, oneline=True, no_renames=True
-        )
-        if status == 0:
-            paths = [f for f in out.rstrip('\0').split('\0') if f]
-            if paths:
-                paths = paths[1:]
+        paths = []
+
+        if len(commits) > 1:
+            # Get a list of changed files for a commit range.
+            start = commits[0].oid + '~'
+            end = commits[-1].oid
+            status, out, _ = git.diff(start, end, z=True, numstat=True, no_renames=True)
+            if status == 0:
+                paths = [f for f in out.rstrip('\0').split('\0') if f]
         else:
-            paths = []
+            # Get the list of changed files in a single commit.
+            commit = commits[0]
+            oid = commit.oid
+            status, out, _ = git.show(
+                oid, z=True, numstat=True, oneline=True, no_renames=True
+            )
+            if status == 0:
+                paths = [f for f in out.rstrip('\0').split('\0') if f]
+                if paths:
+                    paths = paths[1:]  # Skip over the summary on the first line.
+
         self.list_files(paths)
 
     def list_files(self, files_log):
@@ -76,8 +83,8 @@ class FileWidget(TreeWidget):
         if not files_log:
             return
         files = []
-        for f in files_log:
-            item = FileTreeWidgetItem(f)
+        for filename in files_log:
+            item = FileTreeWidgetItem(filename)
             files.append(item)
         self.insertTopLevelItems(0, files)
 
@@ -96,8 +103,8 @@ class FileWidget(TreeWidget):
     def show(self):
         self.adjust_columns(QSize(), QSize())
 
-    def resizeEvent(self, e):
-        self.adjust_columns(e.size(), e.oldSize())
+    def resizeEvent(self, event):
+        self.adjust_columns(event.size(), event.oldSize())
 
     def contextMenuEvent(self, event):
         menu = qtutils.create_menu(N_('Actions'), self)
@@ -108,7 +115,7 @@ class FileWidget(TreeWidget):
         menu.exec_(self.mapToGlobal(event.pos()))
 
     def show_diff(self):
-        self.notifier.notify_observers(DIFFTOOL_SELECTED, self.selected_paths())
+        self.difftool_selected.emit(self.selected_paths())
 
     def _grab_file(self):
         for path in self.selected_paths():
@@ -123,7 +130,7 @@ class FileWidget(TreeWidget):
     def show_history(self):
         items = self.selected_items()
         paths = [i.path for i in items]
-        self.notifier.notify_observers(HISTORIES_SELECTED, paths)
+        self.histories_selected.emit(paths)
 
 
 class FileTreeWidgetItem(QtWidgets.QTreeWidgetItem):
