@@ -1,6 +1,5 @@
-"""This view provides the main git-cola user interface.
-"""
-from __future__ import division, absolute_import, unicode_literals
+"""Main UI for authoring commits and other Git Cola interactions"""
+from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 from functools import partial
 
@@ -37,6 +36,7 @@ from . import browse
 from . import cfgactions
 from . import clone
 from . import commitmsg
+from . import common
 from . import compare
 from . import createbranch
 from . import createtag
@@ -61,16 +61,16 @@ from . import toolbar
 
 class MainView(standard.MainWindow):
     config_actions_changed = Signal(object)
-    updated = Signal()
 
     def __init__(self, context, parent=None):
+        # pylint: disable=too-many-statements,too-many-locals
         standard.MainWindow.__init__(self, parent)
         self.setAttribute(Qt.WA_DeleteOnClose)
 
         self.context = context
         self.git = context.git
         self.dag = None
-        self.model = model = context.model
+        self.model = context.model
         self.prefs_model = prefs_model = prefs.PreferencesModel(context)
         self.toolbar_state = toolbar.ToolBarState(context, self)
 
@@ -85,31 +85,41 @@ class MainView(standard.MainWindow):
             browser = browse.worktree_browser(
                 context, parent=self, show=False, update=False
             )
-            self.browserdock = create_dock(N_('Browser'), self, widget=browser)
+            self.browserdock = create_dock(
+                'Browser', N_('Browser'), self, widget=browser
+            )
 
         # "Actions" widget
+        self.actionswidget = action.ActionButtons(context, self)
         self.actionsdock = create_dock(
-            N_('Actions'), self, widget=action.ActionButtons(context, self)
+            'Actions', N_('Actions'), self, widget=self.actionswidget
         )
         qtutils.hide_dock(self.actionsdock)
 
         # "Repository Status" widget
         self.statusdock = create_dock(
+            'Status',
             N_('Status'),
             self,
-            fn=lambda dock: status.StatusWidget(context, dock.titleBarWidget(), dock),
+            func=lambda dock: status.StatusWidget(context, dock.titleBarWidget(), dock),
         )
         self.statuswidget = self.statusdock.widget()
 
         # "Switch Repository" widgets
         self.bookmarksdock = create_dock(
-            N_('Favorites'), self, fn=lambda dock: bookmarks.bookmark(context, dock)
+            'Favorites',
+            N_('Favorites'),
+            self,
+            func=lambda dock: bookmarks.bookmark(context, dock),
         )
         bookmarkswidget = self.bookmarksdock.widget()
         qtutils.hide_dock(self.bookmarksdock)
 
         self.recentdock = create_dock(
-            N_('Recent'), self, fn=lambda dock: bookmarks.recent(context, dock)
+            'Recent',
+            N_('Recent'),
+            self,
+            func=lambda dock: bookmarks.recent(context, dock),
         )
         recentwidget = self.recentdock.widget()
         qtutils.hide_dock(self.recentdock)
@@ -117,7 +127,10 @@ class MainView(standard.MainWindow):
 
         # "Branch" widgets
         self.branchdock = create_dock(
-            N_('Branches'), self, fn=partial(branch.BranchesWidget, context)
+            'Branches',
+            N_('Branches'),
+            self,
+            func=partial(branch.BranchesWidget, context),
         )
         self.branchwidget = self.branchdock.widget()
         titlebar = self.branchdock.titleBarWidget()
@@ -126,7 +139,10 @@ class MainView(standard.MainWindow):
 
         # "Submodule" widgets
         self.submodulesdock = create_dock(
-            N_('Submodules'), self, fn=partial(submodules.SubmodulesWidget, context)
+            'Submodules',
+            N_('Submodules'),
+            self,
+            func=partial(submodules.SubmodulesWidget, context),
         )
         self.submoduleswidget = self.submodulesdock.widget()
 
@@ -138,24 +154,29 @@ class MainView(standard.MainWindow):
         self.position_label.setFont(font)
 
         # make the position label fixed size to avoid layout issues
-        fm = self.position_label.fontMetrics()
-        width = fm.width('99:999') + defs.spacing
+        metrics = self.position_label.fontMetrics()
+        width = metrics.width('99:999') + defs.spacing
         self.position_label.setMinimumWidth(width)
 
         editor = commitmsg.CommitMessageEditor(context, self)
         self.commiteditor = editor
-        self.commitdock = create_dock(N_('Commit'), self, widget=editor)
+        self.commitdock = create_dock('Commit', N_('Commit'), self, widget=editor)
         titlebar = self.commitdock.titleBarWidget()
         titlebar.add_corner_widget(self.position_label)
 
         # "Console" widget
         self.logwidget = log.LogWidget(context)
-        self.logdock = create_dock(N_('Console'), self, widget=self.logwidget)
+        self.logdock = create_dock(
+            'Console', N_('Console'), self, widget=self.logwidget
+        )
         qtutils.hide_dock(self.logdock)
 
         # "Diff Viewer" widget
         self.diffdock = create_dock(
-            N_('Diff'), self, fn=lambda dock: diff.Viewer(context, parent=dock)
+            'Diff',
+            N_('Diff'),
+            self,
+            func=lambda dock: diff.Viewer(context, parent=dock),
         )
         self.diffviewer = self.diffdock.widget()
         self.diffviewer.set_diff_type(self.model.diff_type)
@@ -174,6 +195,7 @@ class MainView(standard.MainWindow):
             partial(cmds.do, cmds.AmendMode, context),
             False,
         )
+        self.commit_amend_action.setIcon(icons.edit())
         self.commit_amend_action.setShortcut(hotkeys.AMEND)
         self.commit_amend_action.setShortcutContext(Qt.WidgetShortcut)
 
@@ -182,6 +204,11 @@ class MainView(standard.MainWindow):
         )
         self.unstage_all_action.setIcon(icons.remove())
 
+        self.undo_commit_action = add_action(
+            self, N_('Undo Last Commit'), cmds.run(cmds.UndoLastCommit, context)
+        )
+        self.undo_commit_action.setIcon(icons.style_dialog_discard())
+
         self.unstage_selected_action = add_action(
             self, N_('Unstage From Commit'), cmds.run(cmds.UnstageSelected, context)
         )
@@ -190,6 +217,7 @@ class MainView(standard.MainWindow):
         self.show_diffstat_action = add_action(
             self, N_('Diffstat'), self.statuswidget.select_header, hotkeys.DIFFSTAT
         )
+        self.show_diffstat_action.setIcon(icons.diff())
 
         self.stage_modified_action = add_action(
             self,
@@ -210,6 +238,35 @@ class MainView(standard.MainWindow):
         self.apply_patches_action = add_action(
             self, N_('Apply Patches...'), partial(patch.apply_patches, context)
         )
+        self.apply_patches_action.setIcon(icons.diff())
+
+        self.apply_patches_abort_action = add_action(
+            self,
+            N_('Abort Applying Patches...'),
+            cmds.run(cmds.AbortApplyPatch, context),
+        )
+        self.apply_patches_abort_action.setIcon(icons.style_dialog_discard())
+        self.apply_patches_abort_action.setToolTip(
+            N_('Abort the current "git am" patch session')
+        )
+
+        self.apply_patches_continue_action = add_action(
+            self,
+            N_('Continue Applying Patches'),
+            cmds.run(cmds.ApplyPatchesContinue, context),
+        )
+        self.apply_patches_continue_action.setToolTip(
+            N_('Commit the current state and continue applying patches')
+        )
+        self.apply_patches_continue_action.setIcon(icons.commit())
+
+        self.apply_patches_skip_action = add_action(
+            self, N_('Skip Current Patch'), cmds.run(cmds.ApplyPatchesContinue, context)
+        )
+        self.apply_patches_skip_action.setToolTip(
+            N_('Skip applying the current patch and continue applying patches')
+        )
+        self.apply_patches_skip_action.setIcon(icons.discard())
 
         self.export_patches_action = add_action(
             self,
@@ -217,6 +274,7 @@ class MainView(standard.MainWindow):
             partial(guicmds.export_patches, context),
             hotkeys.EXPORT,
         )
+        self.export_patches_action.setIcon(icons.save())
 
         self.new_repository_action = add_action(
             self, N_('New Repository...'), partial(guicmds.open_new_repo, context)
@@ -228,16 +286,18 @@ class MainView(standard.MainWindow):
         )
         self.new_bare_repository_action.setIcon(icons.new())
 
-        prefs_fn = partial(
+        prefs_func = partial(
             prefs_widget.preferences, context, parent=self, model=prefs_model
         )
         self.preferences_action = add_action(
-            self, N_('Preferences'), prefs_fn, QtGui.QKeySequence.Preferences
+            self, N_('Preferences'), prefs_func, QtGui.QKeySequence.Preferences
         )
+        self.preferences_action.setIcon(icons.configure())
 
         self.edit_remotes_action = add_action(
             self, N_('Edit Remotes...'), partial(editremotes.editor, context)
         )
+        self.edit_remotes_action.setIcon(icons.edit())
 
         self.rescan_action = add_action(
             self,
@@ -252,9 +312,8 @@ class MainView(standard.MainWindow):
             N_('Find Files'),
             partial(finder.finder, context),
             hotkeys.FINDER,
-            hotkeys.FINDER_SECONDARY,
         )
-        self.find_files_action.setIcon(icons.zoom_in())
+        self.find_files_action.setIcon(icons.search())
 
         self.browse_recently_modified_action = add_action(
             self,
@@ -262,6 +321,7 @@ class MainView(standard.MainWindow):
             partial(recent.browse_recent_files, context),
             hotkeys.EDIT_SECONDARY,
         )
+        self.browse_recently_modified_action.setIcon(icons.directory())
 
         self.cherry_pick_action = add_action(
             self,
@@ -269,10 +329,16 @@ class MainView(standard.MainWindow):
             partial(guicmds.cherry_pick, context),
             hotkeys.CHERRY_PICK,
         )
+        self.cherry_pick_action.setIcon(icons.cherry_pick())
+        self.cherry_pick_abort_action = add_action(
+            self, N_('Abort Cherry-Pick...'), cmds.run(cmds.AbortCherryPick, context)
+        )
+        self.cherry_pick_abort_action.setIcon(icons.style_dialog_discard())
 
         self.load_commitmsg_action = add_action(
             self, N_('Load Commit Message...'), partial(guicmds.load_commitmsg, context)
         )
+        self.load_commitmsg_action.setIcon(icons.file_text())
 
         self.prepare_commitmsg_hook_action = add_action(
             self,
@@ -284,42 +350,53 @@ class MainView(standard.MainWindow):
         self.save_tarball_action = add_action(
             self, N_('Save As Tarball/Zip...'), partial(archive.save_archive, context)
         )
+        self.save_tarball_action.setIcon(icons.file_zip())
 
         self.quit_action = add_action(self, N_('Quit'), self.close, hotkeys.QUIT)
 
         self.grep_action = add_action(
             self, N_('Grep'), partial(grep.grep, context), hotkeys.GREP
         )
+        self.grep_action.setIcon(icons.search())
 
         self.merge_local_action = add_action(
             self, N_('Merge...'), partial(merge.local_merge, context), hotkeys.MERGE
         )
+        self.merge_local_action.setIcon(icons.merge())
 
         self.merge_abort_action = add_action(
             self, N_('Abort Merge...'), cmds.run(cmds.AbortMerge, context)
         )
+        self.merge_abort_action.setIcon(icons.style_dialog_discard())
 
         self.update_submodules_action = add_action(
             self,
             N_('Update All Submodules...'),
             cmds.run(cmds.SubmodulesUpdate, context),
         )
+        self.update_submodules_action.setIcon(icons.sync())
 
         self.add_submodule_action = add_action(
             self,
             N_('Add Submodule...'),
             partial(submodules.add_submodule, context, parent=self),
         )
+        self.add_submodule_action.setIcon(icons.add())
 
         self.fetch_action = add_action(
             self, N_('Fetch...'), partial(remote.fetch, context), hotkeys.FETCH
         )
+        self.fetch_action.setIcon(icons.download())
+
         self.push_action = add_action(
             self, N_('Push...'), partial(remote.push, context), hotkeys.PUSH
         )
+        self.push_action.setIcon(icons.push())
+
         self.pull_action = add_action(
             self, N_('Pull...'), partial(remote.pull, context), hotkeys.PULL
         )
+        self.pull_action.setIcon(icons.pull())
 
         self.open_repo_action = add_action(
             self, N_('Open...'), partial(guicmds.open_repo, context), hotkeys.OPEN
@@ -336,13 +413,52 @@ class MainView(standard.MainWindow):
         self.stash_action = add_action(
             self, N_('Stash...'), partial(stash.view, context), hotkeys.STASH
         )
+        self.stash_action.setIcon(icons.commit())
 
-        self.reset_branch_head_action = add_action(
-            self, N_('Reset Branch Head'), partial(guicmds.reset_branch_head, context)
+        self.reset_soft_action = add_action(
+            self, N_('Reset Branch (Soft)'), partial(guicmds.reset_soft, context)
         )
+        self.reset_soft_action.setIcon(icons.style_dialog_reset())
+        self.reset_soft_action.setToolTip(cmds.ResetSoft.tooltip('<commit>'))
 
-        self.reset_worktree_action = add_action(
-            self, N_('Reset Worktree'), partial(guicmds.reset_worktree, context)
+        self.reset_mixed_action = add_action(
+            self,
+            N_('Reset Branch and Stage (Mixed)'),
+            partial(guicmds.reset_mixed, context),
+        )
+        self.reset_mixed_action.setIcon(icons.style_dialog_reset())
+        self.reset_mixed_action.setToolTip(cmds.ResetMixed.tooltip('<commit>'))
+
+        self.reset_keep_action = add_action(
+            self,
+            N_('Restore Worktree and Reset All (Keep Unstaged Changes)'),
+            partial(guicmds.reset_keep, context),
+        )
+        self.reset_keep_action.setIcon(icons.style_dialog_reset())
+        self.reset_keep_action.setToolTip(cmds.ResetKeep.tooltip('<commit>'))
+
+        self.reset_merge_action = add_action(
+            self,
+            N_('Restore Worktree and Reset All (Merge)'),
+            partial(guicmds.reset_merge, context),
+        )
+        self.reset_merge_action.setIcon(icons.style_dialog_reset())
+        self.reset_merge_action.setToolTip(cmds.ResetMerge.tooltip('<commit>'))
+
+        self.reset_hard_action = add_action(
+            self,
+            N_('Restore Worktree and Reset All (Hard)'),
+            partial(guicmds.reset_hard, context),
+        )
+        self.reset_hard_action.setIcon(icons.style_dialog_reset())
+        self.reset_hard_action.setToolTip(cmds.ResetHard.tooltip('<commit>'))
+
+        self.restore_worktree_action = add_action(
+            self, N_('Restore Worktree'), partial(guicmds.restore_worktree, context)
+        )
+        self.restore_worktree_action.setIcon(icons.edit())
+        self.restore_worktree_action.setToolTip(
+            cmds.RestoreWorktree.tooltip('<commit>')
         )
 
         self.clone_repo_action = add_action(
@@ -366,42 +482,69 @@ class MainView(standard.MainWindow):
             N_('Visualize Current Branch...'),
             cmds.run(cmds.VisualizeCurrent, context),
         )
+        self.visualize_current_action.setIcon(icons.visualize())
+
         self.visualize_all_action = add_action(
             self, N_('Visualize All Branches...'), cmds.run(cmds.VisualizeAll, context)
         )
+        self.visualize_all_action.setIcon(icons.visualize())
+
         self.search_commits_action = add_action(
             self, N_('Search...'), partial(search.search, context)
         )
+        self.search_commits_action.setIcon(icons.search())
 
         self.browse_branch_action = add_action(
             self,
             N_('Browse Current Branch...'),
             partial(guicmds.browse_current, context),
         )
+        self.browse_branch_action.setIcon(icons.directory())
+
         self.browse_other_branch_action = add_action(
             self, N_('Browse Other Branch...'), partial(guicmds.browse_other, context)
         )
+        self.browse_other_branch_action.setIcon(icons.directory())
+
         self.load_commitmsg_template_action = add_action(
             self,
             N_('Get Commit Message Template'),
             cmds.run(cmds.LoadCommitMessageFromTemplate, context),
         )
+        self.load_commitmsg_template_action.setIcon(icons.style_dialog_apply())
+
         self.help_about_action = add_action(
             self, N_('About'), partial(about.about_dialog, context)
         )
 
+        self.diff_against_commit_action = add_action(
+            self,
+            N_('Against Commit... (Diff Mode)'),
+            partial(guicmds.diff_against_commit, context),
+        )
+        self.diff_against_commit_action.setIcon(icons.compare())
+
+        self.exit_diff_mode_action = add_action(
+            self, N_('Exit Diff Mode'), cmds.run(cmds.ResetMode, context)
+        )
+        self.exit_diff_mode_action.setIcon(icons.compare())
+
         self.diff_expression_action = add_action(
             self, N_('Expression...'), partial(guicmds.diff_expression, context)
         )
+        self.diff_expression_action.setIcon(icons.compare())
+
         self.branch_compare_action = add_action(
             self, N_('Branches...'), partial(compare.compare_branches, context)
         )
+        self.branch_compare_action.setIcon(icons.compare())
 
         self.create_tag_action = add_action(
             self,
             N_('Create Tag...'),
             partial(createtag.create_tag, context),
         )
+        self.create_tag_action.setIcon(icons.tag())
 
         self.create_branch_action = add_action(
             self,
@@ -414,16 +557,19 @@ class MainView(standard.MainWindow):
         self.delete_branch_action = add_action(
             self, N_('Delete...'), partial(guicmds.delete_branch, context)
         )
+        self.delete_branch_action.setIcon(icons.discard())
 
         self.delete_remote_branch_action = add_action(
             self,
             N_('Delete Remote Branch...'),
             partial(guicmds.delete_remote_branch, context),
         )
+        self.delete_remote_branch_action.setIcon(icons.discard())
 
         self.rename_branch_action = add_action(
             self, N_('Rename Branch...'), partial(guicmds.rename_branch, context)
         )
+        self.rename_branch_action.setIcon(icons.edit())
 
         self.checkout_branch_action = add_action(
             self,
@@ -431,9 +577,12 @@ class MainView(standard.MainWindow):
             partial(guicmds.checkout_branch, context),
             hotkeys.CHECKOUT,
         )
+        self.checkout_branch_action.setIcon(icons.branch())
+
         self.branch_review_action = add_action(
             self, N_('Review...'), partial(guicmds.review_branch, context)
         )
+        self.branch_review_action.setIcon(icons.compare())
 
         self.browse_action = add_action(
             self, N_('File Browser...'), partial(browse.worktree_browser, context)
@@ -449,10 +598,12 @@ class MainView(standard.MainWindow):
             cmds.run(cmds.Rebase, context),
             hotkeys.REBASE_START_AND_CONTINUE,
         )
+        self.rebase_start_action.setIcon(icons.play())
 
         self.rebase_edit_todo_action = add_action(
             self, N_('Edit...'), cmds.run(cmds.RebaseEditTodo, context)
         )
+        self.rebase_edit_todo_action.setIcon(icons.edit())
 
         self.rebase_continue_action = add_action(
             self,
@@ -460,14 +611,17 @@ class MainView(standard.MainWindow):
             cmds.run(cmds.RebaseContinue, context),
             hotkeys.REBASE_START_AND_CONTINUE,
         )
+        self.rebase_continue_action.setIcon(icons.play())
 
         self.rebase_skip_action = add_action(
             self, N_('Skip Current Patch'), cmds.run(cmds.RebaseSkip, context)
         )
+        self.rebase_skip_action.setIcon(icons.delete())
 
         self.rebase_abort_action = add_action(
             self, N_('Abort'), cmds.run(cmds.RebaseAbort, context)
         )
+        self.rebase_abort_action.setIcon(icons.close())
 
         # For "Start Rebase" only, reverse the first argument to setEnabled()
         # so that we can operate on it as a group.
@@ -501,6 +655,18 @@ class MainView(standard.MainWindow):
             self, N_('Reset Layout'), self.reset_layout
         )
 
+        self.quick_repository_search = add_action(
+            self,
+            N_('Quick Open...'),
+            lambda: guicmds.open_quick_repo_search(self.context, parent=self),
+            hotkeys.OPEN_REPO_SEARCH,
+        )
+        self.quick_repository_search.setIcon(icons.search())
+
+        self.terminal_action = common.terminal_action(
+            context, self, hotkey=hotkeys.TERMINAL
+        )
+
         # Create the application menu
         self.menubar = QtWidgets.QMenuBar(self)
         self.setMenuBar(self.menubar)
@@ -508,6 +674,7 @@ class MainView(standard.MainWindow):
         # File Menu
         add_menu = qtutils.add_menu
         self.file_menu = add_menu(N_('&File'), self.menubar)
+        self.file_menu.addAction(self.quick_repository_search)
         # File->Open Recent menu
         self.open_recent_menu = self.file_menu.addMenu(N_('Open Recent'))
         self.open_recent_menu.setIcon(icons.folder())
@@ -523,10 +690,16 @@ class MainView(standard.MainWindow):
         self.file_menu.addAction(self.edit_remotes_action)
         self.file_menu.addAction(self.browse_recently_modified_action)
         self.file_menu.addSeparator()
-        self.file_menu.addAction(self.apply_patches_action)
-        self.file_menu.addAction(self.export_patches_action)
         self.file_menu.addAction(self.save_tarball_action)
 
+        self.patches_menu = self.file_menu.addMenu(N_('Patches'))
+        self.patches_menu.setIcon(icons.diff())
+        self.patches_menu.addAction(self.export_patches_action)
+        self.patches_menu.addAction(self.apply_patches_action)
+        self.patches_menu.addAction(self.apply_patches_continue_action)
+        self.patches_menu.addAction(self.apply_patches_skip_action)
+        self.patches_menu.addAction(self.apply_patches_abort_action)
+
         # Git Annex / Git LFS
         annex = core.find_executable('git-annex')
         lfs = core.find_executable('git-lfs')
@@ -554,27 +727,36 @@ class MainView(standard.MainWindow):
             bookmarkswidget.tree,
             recentwidget.tree,
         )
+        select_widgets = copy_widgets + (self.statuswidget.tree,)
         edit_proxy.override('copy', copy_widgets)
-        edit_proxy.override('selectAll', copy_widgets)
+        edit_proxy.override('selectAll', select_widgets)
 
         edit_menu = self.edit_menu = add_menu(N_('&Edit'), self.menubar)
-        add_action(edit_menu, N_('Undo'), edit_proxy.undo, hotkeys.UNDO)
-        add_action(edit_menu, N_('Redo'), edit_proxy.redo, hotkeys.REDO)
+        undo = add_action(edit_menu, N_('Undo'), edit_proxy.undo, hotkeys.UNDO)
+        undo.setIcon(icons.undo())
+        redo = add_action(edit_menu, N_('Redo'), edit_proxy.redo, hotkeys.REDO)
+        redo.setIcon(icons.redo())
         edit_menu.addSeparator()
-        add_action(edit_menu, N_('Cut'), edit_proxy.cut, hotkeys.CUT)
-        add_action(edit_menu, N_('Copy'), edit_proxy.copy, hotkeys.COPY)
-        add_action(edit_menu, N_('Paste'), edit_proxy.paste, hotkeys.PASTE)
-        add_action(edit_menu, N_('Delete'), edit_proxy.delete, hotkeys.DELETE)
+        cut = add_action(edit_menu, N_('Cut'), edit_proxy.cut, hotkeys.CUT)
+        cut.setIcon(icons.cut())
+        copy = add_action(edit_menu, N_('Copy'), edit_proxy.copy, hotkeys.COPY)
+        copy.setIcon(icons.copy())
+        paste = add_action(edit_menu, N_('Paste'), edit_proxy.paste, hotkeys.PASTE)
+        paste.setIcon(icons.paste())
+        delete = add_action(edit_menu, N_('Delete'), edit_proxy.delete, hotkeys.DELETE)
+        delete.setIcon(icons.delete())
         edit_menu.addSeparator()
-        add_action(
+        select_all = add_action(
             edit_menu, N_('Select All'), edit_proxy.selectAll, hotkeys.SELECT_ALL
         )
+        select_all.setIcon(icons.select_all())
         edit_menu.addSeparator()
-
-        commitmsg.add_menu_actions(edit_menu, self.commiteditor.menu_actions)
+        qtutils.add_menu_actions(edit_menu, self.commiteditor.menu_actions)
 
         # Actions menu
         self.actions_menu = add_menu(N_('Actions'), self.menubar)
+        if self.terminal_action is not None:
+            self.actions_menu.addAction(self.terminal_action)
         self.actions_menu.addAction(self.fetch_action)
         self.actions_menu.addAction(self.push_action)
         self.actions_menu.addAction(self.pull_action)
@@ -582,16 +764,13 @@ class MainView(standard.MainWindow):
         self.actions_menu.addSeparator()
         self.actions_menu.addAction(self.create_tag_action)
         self.actions_menu.addAction(self.cherry_pick_action)
+        self.actions_menu.addAction(self.cherry_pick_abort_action)
         self.actions_menu.addAction(self.merge_local_action)
         self.actions_menu.addAction(self.merge_abort_action)
         self.actions_menu.addSeparator()
         self.actions_menu.addAction(self.update_submodules_action)
         self.actions_menu.addAction(self.add_submodule_action)
         self.actions_menu.addSeparator()
-        self.actions_reset_menu = self.actions_menu.addMenu(N_('Reset'))
-        self.actions_reset_menu.addAction(self.reset_branch_head_action)
-        self.actions_reset_menu.addAction(self.reset_worktree_action)
-        self.actions_menu.addSeparator()
         self.actions_menu.addAction(self.grep_action)
         self.actions_menu.addAction(self.search_commits_action)
 
@@ -600,7 +779,10 @@ class MainView(standard.MainWindow):
         self.commit_menu.setTitle(N_('Commit@@verb'))
         self.commit_menu.addAction(self.commiteditor.commit_action)
         self.commit_menu.addAction(self.commit_amend_action)
+        self.commit_menu.addAction(self.undo_commit_action)
         self.commit_menu.addSeparator()
+        self.commit_menu.addAction(self.statuswidget.tree.process_selection_action)
+        self.commit_menu.addAction(self.statuswidget.tree.stage_or_unstage_all_action)
         self.commit_menu.addAction(self.stage_modified_action)
         self.commit_menu.addAction(self.stage_untracked_action)
         self.commit_menu.addSeparator()
@@ -615,8 +797,10 @@ class MainView(standard.MainWindow):
         self.diff_menu = add_menu(N_('Diff'), self.menubar)
         self.diff_menu.addAction(self.diff_expression_action)
         self.diff_menu.addAction(self.branch_compare_action)
-        self.diff_menu.addSeparator()
         self.diff_menu.addAction(self.show_diffstat_action)
+        self.diff_menu.addSeparator()
+        self.diff_menu.addAction(self.diff_against_commit_action)
+        self.diff_menu.addAction(self.exit_diff_mode_action)
 
         # Branch Menu
         self.branch_menu = add_menu(N_('Branch'), self.menubar)
@@ -635,7 +819,7 @@ class MainView(standard.MainWindow):
         self.branch_menu.addAction(self.visualize_all_action)
 
         # Rebase menu
-        self.rebase_menu = add_menu(N_('Rebase'), self.actions_menu)
+        self.rebase_menu = add_menu(N_('Rebase'), self.menubar)
         self.rebase_menu.addAction(self.rebase_start_action)
         self.rebase_menu.addAction(self.rebase_edit_todo_action)
         self.rebase_menu.addSeparator()
@@ -644,14 +828,26 @@ class MainView(standard.MainWindow):
         self.rebase_menu.addSeparator()
         self.rebase_menu.addAction(self.rebase_abort_action)
 
+        # Reset menu
+        self.reset_menu = add_menu(N_('Reset'), self.menubar)
+        self.reset_menu.addAction(self.unstage_all_action)
+        self.reset_menu.addAction(self.undo_commit_action)
+        self.reset_menu.addSeparator()
+        self.reset_menu.addAction(self.reset_soft_action)
+        self.reset_menu.addAction(self.reset_mixed_action)
+        self.reset_menu.addAction(self.restore_worktree_action)
+        self.reset_menu.addSeparator()
+        self.reset_menu.addAction(self.reset_keep_action)
+        self.reset_menu.addAction(self.reset_merge_action)
+        self.reset_menu.addAction(self.reset_hard_action)
+
         # View Menu
         self.view_menu = add_menu(N_('View'), self.menubar)
         # pylint: disable=no-member
         self.view_menu.aboutToShow.connect(lambda: self.build_view_menu(self.view_menu))
         self.setup_dockwidget_view_menu()
         if utils.is_darwin():
-            # TODO or self.menubar.setNativeMenuBar(False)
-            # Since native OSX menu doesn't show empty entries
+            # The native macOS menu doesn't show empty entries.
             self.build_view_menu(self.view_menu)
 
         # Help Menu
@@ -686,13 +882,13 @@ class MainView(standard.MainWindow):
         self.tabifyDockWidget(self.actionsdock, self.logdock)
 
         # Listen for model notifications
-        model.add_observer(model.message_updated, self.updated.emit)
-        model.add_observer(model.message_mode_changed, lambda mode: self.updated.emit())
-
-        prefs_model.add_observer(
-            prefs_model.message_config_updated, self._config_updated
+        self.model.updated.connect(self.refresh, type=Qt.QueuedConnection)
+        self.model.mode_changed.connect(
+            lambda mode: self.refresh(), type=Qt.QueuedConnection
         )
 
+        prefs_model.config_updated.connect(self._config_updated)
+
         # Set a default value
         self.show_cursor_position(1, 0)
 
@@ -707,10 +903,13 @@ class MainView(standard.MainWindow):
         self.commiteditor.up.connect(self.statuswidget.move_up)
         self.commiteditor.down.connect(self.statuswidget.move_down)
 
-        self.updated.connect(self.refresh, type=Qt.QueuedConnection)
-
         self.config_actions_changed.connect(
-            self._install_config_actions, type=Qt.QueuedConnection
+            lambda names_and_shortcuts: _install_config_actions(
+                context,
+                self.actions_menu,
+                names_and_shortcuts,
+            ),
+            type=Qt.QueuedConnection,
         )
         self.init_state(context.settings, self.set_initial_size)
 
@@ -759,6 +958,8 @@ class MainView(standard.MainWindow):
         """Save state in the settings"""
         commit_msg = self.commiteditor.commit_message(raw=True)
         self.model.save_commitmsg(msg=commit_msg)
+        for browser in list(self.context.browser_windows):
+            browser.close()
         standard.MainWindow.closeEvent(self, event)
 
     def create_view_menu(self):
@@ -777,12 +978,12 @@ class MainView(standard.MainWindow):
             menu_action.setParent(menu)
             menu.addAction(menu_action)
 
-        menu.addSeparator()
         context = self.context
         menu_action = menu.addAction(
-            N_('Add Toolbar'), partial(toolbar.add_toolbar, context, self)
+            N_('New Toolbar'), partial(toolbar.add_toolbar, context, self)
         )
         menu_action.setIcon(icons.add())
+        menu.addSeparator()
 
         dockwidgets = [
             self.logdock,
@@ -875,33 +1076,22 @@ class MainView(standard.MainWindow):
     def start(self, context):
         """Do the expensive "get_config_actions()" call in the background"""
         # Install .git-config-defined actions
-        task = qtutils.SimpleTask(self, self.get_config_actions)
+        task = qtutils.SimpleTask(self.get_config_actions)
         context.runtask.start(task)
 
     def get_config_actions(self):
         actions = cfgactions.get_config_actions(self.context)
         self.config_actions_changed.emit(actions)
 
-    def _install_config_actions(self, names_and_shortcuts):
-        """Install .gitconfig-defined actions"""
-        if not names_and_shortcuts:
-            return
-        context = self.context
-        menu = self.actions_menu
-        menu.addSeparator()
-        for (name, shortcut) in names_and_shortcuts:
-            callback = cmds.run(cmds.RunConfigAction, context, name)
-            menu_action = menu.addAction(name, callback)
-            if shortcut:
-                menu_action.setShortcut(shortcut)
-
     def refresh(self):
         """Update the title with the current branch and directory name."""
         curbranch = self.model.currentbranch
-        curdir = core.getcwd()
         is_merging = self.model.is_merging
         is_rebasing = self.model.is_rebasing
+        is_applying_patch = self.model.is_applying_patch
+        is_cherry_picking = self.model.is_rebasing
 
+        curdir = core.getcwd()
         msg = N_('Repository: %s') % curdir
         msg += '\n'
         msg += N_('Branch: %s') % curbranch
@@ -913,7 +1103,18 @@ class MainView(standard.MainWindow):
                 'Resolve conflicts, commit changes, and run:\n'
                 '    Rebase > Continue'
             )
-
+        elif is_applying_patch:
+            msg += '\n\n'
+            msg += N_(
+                'This repository has unresolved conflicts after applying a patch.\n'
+                'Resolve conflicts and commit changes.'
+            )
+        elif is_cherry_picking:
+            msg += '\n\n'
+            msg += N_(
+                'This repository is in the middle of a cherry-pick.\n'
+                'Resolve conflicts and commit changes.'
+            )
         elif is_merging:
             msg += '\n\n'
             msg += N_(
@@ -929,7 +1130,11 @@ class MainView(standard.MainWindow):
             self.commit_amend_action.setChecked(False)
 
         self.commitdock.setToolTip(msg)
+
+        self.actionswidget.set_mode(self.mode)
         self.commiteditor.set_mode(self.mode)
+        self.statuswidget.set_mode(self.mode)
+
         self.update_actions()
 
     def refresh_window_title(self):
@@ -938,18 +1143,28 @@ class MainView(standard.MainWindow):
 
         project = self.model.project
         curbranch = self.model.currentbranch
+        is_cherry_picking = self.model.is_cherry_picking
         is_merging = self.model.is_merging
         is_rebasing = self.model.is_rebasing
+        is_applying_patch = self.model.is_applying_patch
+        is_diff_mode = self.model.is_diff_mode()
+        is_amend_mode = self.mode == self.model.mode_amend
+
         prefix = uchr(0xAB)
         suffix = uchr(0xBB)
 
-        if is_rebasing:
-            alerts.append(N_('Rebasing'))
+        if is_amend_mode:
+            alerts.append(N_('Amending'))
+        elif is_diff_mode:
+            alerts.append(N_('Diff Mode'))
+        elif is_cherry_picking:
+            alerts.append(N_('Cherry-picking'))
         elif is_merging:
             alerts.append(N_('Merging'))
-
-        if self.mode == self.model.mode_amend:
-            alerts.append(N_('Amending'))
+        elif is_rebasing:
+            alerts.append(N_('Rebasing'))
+        elif is_applying_patch:
+            alerts.append(N_('Applying Patch'))
 
         if alerts:
             alert_text = (prefix + ' %s ' + suffix + ' ') % ', '.join(alerts)
@@ -974,6 +1189,14 @@ class MainView(standard.MainWindow):
 
         self.annex_init_action.setEnabled(not self.model.annex)
         self.lfs_init_action.setEnabled(not self.model.lfs)
+        self.merge_abort_action.setEnabled(self.model.is_merging)
+        self.cherry_pick_abort_action.setEnabled(self.model.is_cherry_picking)
+        self.apply_patches_continue_action.setEnabled(self.model.is_applying_patch)
+        self.apply_patches_skip_action.setEnabled(self.model.is_applying_patch)
+        self.apply_patches_abort_action.setEnabled(self.model.is_applying_patch)
+
+        diff_mode = self.model.mode == self.model.mode_diff
+        self.exit_diff_mode_action.setEnabled(diff_mode)
 
     def update_menu_actions(self):
         # Enable the Prepare Commit Message action if the hook exists
@@ -1077,7 +1300,7 @@ class MainView(standard.MainWindow):
         self.dag = dag.git_dag(self.context, existing_view=self.dag)
 
     def show_cursor_position(self, rows, cols):
-        display = '%02d:%02d' % (rows, cols)
+        display_content = '%02d:%02d' % (rows, cols)
         css = """
             <style>
             .good {
@@ -1105,7 +1328,7 @@ class MainView(standard.MainWindow):
             cls = 'first-warning'
         else:
             cls = 'good'
-        div = '<div class="%s">%s</div>' % (cls, display)
+        div = '<div class="%s">%s</div>' % (cls, display_content)
         self.position_label.setText(css + div)
 
 
@@ -1139,9 +1362,9 @@ class FocusProxy(object):
 
         def callback():
             focus = self.focus(name)
-            fn = getattr(focus, name, None)
-            if fn:
-                fn()
+            func = getattr(focus, name, None)
+            if func:
+                func()
 
         return callback
 
@@ -1164,3 +1387,55 @@ def focus_dock(dockwidget):
         show_dock(dockwidget)
     else:
         dockwidget.toggleViewAction().trigger()
+
+
+def _install_config_actions(context, menu, names_and_shortcuts):
+    """Install .gitconfig-defined actions"""
+    if not names_and_shortcuts:
+        return
+    menu.addSeparator()
+    cache = {}
+    for (name, shortcut) in names_and_shortcuts:
+        sub_menu, action_name = build_menus(name, menu, cache)
+        callback = cmds.run(cmds.RunConfigAction, context, name)
+        menu_action = sub_menu.addAction(action_name, callback)
+        if shortcut:
+            menu_action.setShortcut(shortcut)
+
+
+def build_menus(name, menu, cache):
+    """Create a chain of QMenu entries parented under a root QMenu
+
+    A name of "a/b/c" create a menu chain of menu -> QMenu("a") -> QMenu("b")
+    and returns a tuple of (QMenu("b"), "c").
+
+    :param name: The full entry path, ex: "a/b/c" where "a/b" is the menu chain.
+    :param menu: The root menu under which to create the menu chain.
+    :param cache: A dict cache of previously created menus to avoid duplicates.
+
+    """
+    # NOTE: utils.split() and friends are used instead of os.path.split() because
+    # slash '/' is the only supported "<menu>/name" separator.  Use of os.path.split()
+    # would introduce differences in behavior across platforms.
+
+    # If the menu_path is empty then no parent menus need to be created.
+    # The action will be added to the root menu.
+    menu_path, text = utils.split(utils.normalize_slash(name))
+
+    if not menu_path:
+        return (menu, text)
+    # When menu_path contains ex: "a/b" we will create two menus: "a" and "b".
+    # The root menu is the parent of "a" and "a" is the parent of "b".
+    # The menu returned to the caller is "b".
+    #
+    # Loop over the individual menu basenames alongside the full subpath returned by
+    # pathset(). The subpath is a cache key for finding previously created menus.
+    menu_names = utils.splitpath(menu_path)  # ['a', 'b']
+    menu_pathset = utils.pathset(menu_path)  # ['a', 'a/b']
+    for menu_name, menu_id in zip(menu_names, menu_pathset):
+        try:
+            menu = cache[menu_id]
+        except KeyError:
+            menu = cache[menu_id] = menu.addMenu(menu_name)
+
+    return (menu, text)
