@@ -1,4 +1,4 @@
-from __future__ import division, absolute_import, unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
 import os
 import re
 
@@ -10,7 +10,6 @@ from ..qtutils import get
 from .. import core
 from .. import cmds
 from .. import hotkeys
-from .. import observable
 from .. import icons
 from .. import qtutils
 from .standard import Dialog
@@ -94,8 +93,7 @@ class ApplyPatches(Dialog):
         # pylint: disable=no-member
         self.tree.itemSelectionChanged.connect(self._tree_selection_changed)
 
-        self.notifier = notifier = observable.Observable()
-        self.diffwidget = diff.DiffWidget(context, notifier, self, is_commit=True)
+        self.diffwidget = diff.DiffWidget(context, self, is_commit=True)
 
         self.add_button = qtutils.create_toolbutton(
             text=N_('Add'), icon=icons.add(), tooltip=N_('Add patches (+)')
@@ -136,8 +134,8 @@ class ApplyPatches(Dialog):
         self.bottom_layout = qtutils.hbox(
             defs.no_margin,
             defs.button_spacing,
-            self.close_button,
             qtutils.STRETCH,
+            self.close_button,
             self.apply_button,
         )
 
