@@ -1,6 +1,6 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
-import time
 from functools import partial
+import time
 
 from qtpy import QtCore
 from qtpy import QtGui
@@ -25,6 +25,8 @@ from . import defs
 class WidgetMixin(object):
     """Mix-in for common utilities and serialization of widget state"""
 
+    closed = Signal(QtWidgets.QWidget)
+
     def __init__(self):
         self._unmaximized_rect = {}
 
@@ -121,6 +123,7 @@ class WidgetMixin(object):
 
     def closeEvent(self, event):
         self.save_settings()
+        self.closed.emit(self)
         self.Base.closeEvent(self, event)
 
     def init_size(self, parent=None, settings=None, width=0, height=0):
@@ -205,7 +208,7 @@ class MainWindowMixin(WidgetMixin):
 
     def update_dockwidget_lock_state(self):
         if self.lock_layout:
-            features = QDockWidget.DockWidgetClosable | QDockWidget.DockWidgetFloatable
+            features = QDockWidget.DockWidgetClosable
         else:
             features = (
                 QDockWidget.DockWidgetClosable
@@ -300,23 +303,9 @@ class TreeMixin(object):
         was_collapsed = not was_expanded
 
         # Vim keybindings...
-        # Rewrite the event before marshalling to QTreeView.event()
-        key = event.key()
+        event = _create_vim_navigation_key_event(event)
 
-        # Remap 'H' to 'Left'
-        if key == Qt.Key_H:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Left, event.modifiers())
-        # Remap 'J' to 'Down'
-        elif key == Qt.Key_J:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Down, event.modifiers())
-        # Remap 'K' to 'Up'
-        elif key == Qt.Key_K:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Up, event.modifiers())
-        # Remap 'L' to 'Right'
-        elif key == Qt.Key_L:
-            event = QtGui.QKeyEvent(event.type(), Qt.Key_Right, event.modifiers())
-
-        # Re-read the event key to take the remappings into account
+        # Read the updated event key to take the remappings into account
         key = event.key()
         if key == Qt.Key_Up:
             idxs = widget.selectedIndexes()
@@ -343,14 +332,12 @@ class TreeMixin(object):
 
         # Process non-root entries with valid parents only.
         elif key == Qt.Key_Left and index.parent().isValid():
-
             # File entries have rowCount() == 0
             model = widget.model()
-            if (
-                hasattr(model, 'itemFromIndex')
-                and model.itemFromIndex(index).rowCount() == 0
-            ):
-                widget.setCurrentIndex(index.parent())
+            if hasattr(model, 'itemFromIndex'):
+                item = model.itemFromIndex(index)
+                if hasattr(item, 'rowCount') and item.rowCount() == 0:
+                    widget.setCurrentIndex(index.parent())
 
             # Otherwise, do this for collapsed directories only
             elif was_collapsed:
@@ -386,12 +373,11 @@ class TreeMixin(object):
         widget = self.widget
         if hasattr(widget, 'selectedItems'):
             return widget.selectedItems()
+        if hasattr(widget, 'itemFromIndex'):
+            item_from_index = widget.itemFromIndex
         else:
-            if hasattr(widget, 'itemFromIndex'):
-                item_from_index = widget.itemFromIndex
-            else:
-                item_from_index = widget.model().itemFromIndex
-            return [item_from_index(i) for i in widget.selectedIndexes()]
+            item_from_index = widget.model().itemFromIndex
+        return [item_from_index(i) for i in widget.selectedIndexes()]
 
     def selected_item(self):
         """Return the first selected item"""
@@ -428,6 +414,24 @@ class TreeMixin(object):
                 widget.setColumnWidth(idx, value)
 
 
+def _create_vim_navigation_key_event(event):
+    """Support minimal Vim-like keybindings by rewriting the QKeyEvents"""
+    key = event.key()
+    # Remap 'H' to 'Left'
+    if key == Qt.Key_H:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Left, event.modifiers())
+    # Remap 'J' to 'Down'
+    elif key == Qt.Key_J:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Down, event.modifiers())
+    # Remap 'K' to 'Up'
+    elif key == Qt.Key_K:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Up, event.modifiers())
+    # Remap 'L' to 'Right'
+    elif key == Qt.Key_L:
+        event = QtGui.QKeyEvent(event.type(), Qt.Key_Right, event.modifiers())
+    return event
+
+
 class DraggableTreeMixin(TreeMixin):
     """A tree widget with internal drag+drop reordering of rows
 
@@ -522,7 +526,6 @@ class Dialog(WidgetMixin, QtWidgets.QDialog):
         self.dispose()
         return self.Base.reject(self)
 
-    # pylint: disable=no-self-use
     def dispose(self):
         """Extension method for model deregistration in sub-classes"""
         return
@@ -644,40 +647,45 @@ class ProgressDialog(QtWidgets.QProgressDialog):
 
     def __init__(self, title, label, parent):
         QtWidgets.QProgressDialog.__init__(self, parent)
+        self._parent = parent
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
+
+        self.animation_thread = ProgressAnimationThread(label, self)
+        self.animation_thread.updated.connect(self.set_text, type=Qt.QueuedConnection)
+
         self.reset()
         self.setRange(0, 0)
         self.setMinimumDuration(0)
         self.setCancelButton(None)
         self.setFont(qtutils.default_monospace_font())
-        self.thread = ProgressAnimationThread(label, self)
-        self.thread.updated.connect(self.refresh, type=Qt.QueuedConnection)
-
         self.set_details(title, label)
 
     def set_details(self, title, label):
+        """Update the window title and progress label"""
         self.setWindowTitle(title)
         self.setLabelText(label + '     ')
-        self.thread.set_text(label)
+        self.animation_thread.set_text(label)
 
-    def refresh(self, txt):
+    def set_text(self, txt):
+        """Set the label text"""
         self.setLabelText(txt)
 
     def keyPressEvent(self, event):
+        """Customize keyPressEvent to remove the ESC key cancel feature"""
         if event.key() != Qt.Key_Escape:
             super(ProgressDialog, self).keyPressEvent(event)
 
-    def show(self):
+    def start(self):
+        """Start the animation thread and use a wait cursor"""
         QtWidgets.QApplication.setOverrideCursor(Qt.WaitCursor)
-        super(ProgressDialog, self).show()
-        self.thread.start()
+        self.animation_thread.start()
 
-    def hide(self):
+    def stop(self):
+        """Stop the animation thread and restore the normal cursor"""
+        self.animation_thread.stop()
+        self.animation_thread.wait()
         QtWidgets.QApplication.restoreOverrideCursor()
-        self.thread.stop()
-        self.thread.wait()
-        super(ProgressDialog, self).hide()
 
 
 class ProgressAnimationThread(QtCore.QThread):
@@ -897,6 +905,32 @@ class MessageBox(Dialog):
         self.show()
         return self.exec_()
 
+    def apply_state(self, state):
+        """Imports data for view save/restore"""
+        desktop_width, desktop_height = qtutils.desktop_size()
+        width = min(desktop_width, utils.asint(state.get('width')))
+        height = min(desktop_height, utils.asint(state.get('height')))
+        x = min(desktop_width, utils.asint(state.get('x')))
+        y = min(desktop_height, utils.asint(state.get('y')))
+        result = False
+
+        if width and height:
+            self.resize(width, height)
+            self.move(x, y)
+            result = True
+
+        return result
+
+    def export_state(self):
+        """Exports data for view save/restore"""
+        desktop_width, desktop_height = qtutils.desktop_size()
+        state = {}
+        state['width'] = min(desktop_width, self.width())
+        state['height'] = min(desktop_height, self.height())
+        state['x'] = min(desktop_width, self.x())
+        state['y'] = min(desktop_height, self.y())
+        return state
+
 
 def confirm(
     title,
@@ -993,8 +1027,7 @@ def save_as(filename, title):
 
 
 def async_command(title, cmd, runtask):
-    parent = qtutils.active_window()
-    task = qtutils.SimpleTask(parent, partial(core.run_command, cmd))
+    task = qtutils.SimpleTask(partial(core.run_command, cmd))
     task.connect(partial(async_command_result, title, cmd))
     runtask.start(task)
 
